__author__ = 'churas'

import unittest
import tempfile
import os.path
import stat

"""
test_postevaluation
--------------------------------

Tests for `postevaluation` module.
"""
import shutil
import os

from mock import Mock

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.evaluation import EvaluationTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.postevaluation import PostEvaluationTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.task import SmtpEmailer
from d3r.celpp.postevaluation import PostEvaluationEmailer


class TestPostEvaluation(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = PostEvaluationTask(temp_dir, params)

            # try with no dir
            self.assertEqual(task.get_uploadable_files(), [])

            # try with empty dir
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # try with final log
            final_log = os.path.join(task.get_dir(),
                                     EvaluationTask.FINAL_LOG)
            open(final_log, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            self.assertTrue(final_log in flist)

            # try with csv file
            csvfile = os.path.join(task.get_dir(),
                                   'Overall_RMSD_foo' +
                                   PostEvaluationTask.CSV_SUFFIX)
            open(csvfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            self.assertTrue(csvfile in flist)
            self.assertTrue(final_log in flist)

            # try with summary.txt file
            open(task.get_summary_txt(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            self.assertTrue(task.get_summary_txt() in flist)
            self.assertTrue(final_log in flist)

            # try with 2 csv file
            csvfile2 = os.path.join(task.get_dir(),
                                    'Overall_RMSD_foo2' +
                                    PostEvaluationTask.CSV_SUFFIX)
            open(csvfile2, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 4)
            self.assertTrue(csvfile in flist)
            self.assertTrue(csvfile2 in flist)
            self.assertTrue(task.get_summary_txt() in flist)
            self.assertTrue(final_log in flist)
        finally:
            shutil.rmtree(temp_dir)

    def test_postevaluationtask_constructor(self):
        params = D3RParameters()
        task = PostEvaluationTask('/blah', params)
        self.assertEquals(task.get_name(), PostEvaluationTask.TASK_NAME)
        self.assertEquals(task.get_stage(), 8)
        self.assertEqual(task.get_status(), D3RTask.UNKNOWN_STATUS)

    def test_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            # post evaluation task exists already
            task = PostEvaluationTask(temp_dir, params)
            task.create_dir()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             task.get_dir_name() +
                             ' already exists and status is unknown')

            # post evaluation task already complete
            task = PostEvaluationTask(temp_dir, params)

            open(os.path.join(task.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_summary_txt(self):
        params = D3RParameters()

        task = PostEvaluationTask('/foo', params)
        self.assertEqual(task.get_summary_txt(),
                         os.path.join(task.get_dir(),
                                      PostEvaluationTask.SUMMARY_TXT))

    def test_get_postevaluation_summary(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = PostEvaluationTask(temp_dir, params)
            task.create_dir()

            # no file
            res = task.get_postevaluation_summary()
            self.assertEqual(res, '\n\nNo ' + task.get_summary_txt() +
                             ' file found\n')

            # empty file
            s_file = os.path.join(task.get_dir(),
                                  PostEvaluationTask.SUMMARY_TXT)
            open(s_file, 'a').close()
            res = task.get_postevaluation_summary()
            self.assertEqual(res, '\n\n\n')

            os.chmod(task.get_summary_txt(), 0)
            res = task.get_postevaluation_summary()
            self.assertTrue('\n\nUnable to generate postevaluation' in res)

            os.chmod(task.get_summary_txt(), stat.S_IRWXU)

            # some data in the file
            f = open(s_file, 'w')
            f.write('hello\nhi\n')
            f.flush()
            f.close()
            res = task.get_postevaluation_summary()
            self.assertEqual(res, '\n\nhello\nhi\n\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_all_csv_files_in_dir(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = PostEvaluationTask(temp_dir, params)
            task.create_dir()

            # test empty dir
            res = task.get_all_csv_files_in_dir()
            self.assertEqual(len(res), 0)

            # add directory with csv ending
            csvdir = os.path.join(task.get_dir(), 'hi ' +
                                  PostEvaluationTask.CSV_SUFFIX)
            os.makedirs(csvdir, mode=0o755)
            res = task.get_all_csv_files_in_dir()
            self.assertEqual(len(res), 0)

            # 1 csv
            onecsv = os.path.join(task.get_dir(), 'Overall_RMSD_hi' +
                                  PostEvaluationTask.CSV_SUFFIX)
            open(onecsv, 'a').close()
            res = task.get_all_csv_files_in_dir()
            self.assertEqual(len(res), 1)
            self.assertTrue(onecsv in res)

            # 3 csv
            twocsv = os.path.join(task.get_dir(), 'Overall_RMSD_bye' +
                                  PostEvaluationTask.CSV_SUFFIX)
            open(twocsv, 'a').close()
            threecsv = os.path.join(task.get_dir(), 'some' +
                                    PostEvaluationTask.CSV_SUFFIX)
            open(threecsv, 'a').close()

            res = task.get_all_csv_files_in_dir()
            self.assertEqual(len(res), 3)
            self.assertTrue(onecsv in res)
            self.assertTrue(twocsv in res)
            self.assertTrue(threecsv in res)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_all_evaluation_tasks(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            # no evaluation tasks
            task = PostEvaluationTask(temp_dir, params)
            res = task.get_all_evaluation_tasks()
            self.assertEqual(len(res), 0)

            # directory in path that does NOT match suffix
            task.create_dir()
            os.makedirs(os.path.join(temp_dir, 'stage.x.1'), mode=0o755)
            res = task.get_all_evaluation_tasks()
            self.assertEqual(len(res), 0)

            # one task, but is file (weird case)
            weirdtaskfile = os.path.join(temp_dir, 'stage.7.hi.' +
                                         EvaluationTaskFactory.SCORING_SUFFIX)
            open(weirdtaskfile, 'a').close()
            res = task.get_all_evaluation_tasks()
            self.assertEqual(len(res), 0)
            self.assertEqual(task.get_email_log(),
                             'Just a note, found a task with valid name, but '
                             'it is not a directory ' + weirdtaskfile)
            # one task
            etask = EvaluationTask(temp_dir, 'foo.evaluation',
                                   None, params)
            etask.create_dir()
            res = task.get_all_evaluation_tasks()
            self.assertEqual(len(res), 1)
            self.assertTrue(res[0].get_dir_name(), etask.get_dir_name())

            # three tasks two with non complete status
            etask2 = EvaluationTask(temp_dir,
                                    'bla_dock.extsubmission.evaluation',
                                    None, params)
            etask2.create_dir()
            open(os.path.join(etask2.get_dir(),
                              D3RTask.ERROR_FILE), 'a').close()

            etask3 = EvaluationTask(temp_dir,
                                    '12_x.extsubmission.evaluation',
                                    None, params)
            etask3.create_dir()
            open(os.path.join(etask3.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            res = task.get_all_evaluation_tasks()
            self.assertEqual(len(res), 3)
            d_name_list = []
            for entry in res:
                d_name_list.append(entry.get_dir_name())

            self.assertTrue(etask.get_dir_name() in d_name_list)
            self.assertTrue(etask2.get_dir_name() in d_name_list)
            self.assertTrue(etask3.get_dir_name() in d_name_list)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_evaluationdir_args(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            # no evaluation tasks
            task = PostEvaluationTask(temp_dir, params)
            res = task._get_evaluationdir_args()
            self.assertEqual(res, '')

            # one task
            etask = EvaluationTask(temp_dir, 'foo.evaluation',
                                   None, params)
            etask.create_dir()
            res = task._get_evaluationdir_args()
            self.assertEqual(res, ' ' + PostEvaluationTask.EVALUATIONDIR_ARG +
                             ' ' + etask.get_dir())

            # three tasks
            etask2 = EvaluationTask(temp_dir,
                                    'bla_dock.extsubmission.evaluation',
                                    None, params)
            etask2.create_dir()
            open(os.path.join(etask2.get_dir(),
                              D3RTask.ERROR_FILE), 'a').close()

            etask3 = EvaluationTask(temp_dir,
                                    '12_x.extsubmission.evaluation',
                                    None, params)
            etask3.create_dir()
            cfile = os.path.join(etask3.get_dir(), D3RTask.COMPLETE_FILE)
            open(cfile, 'a').close()

            res = task._get_evaluationdir_args()
            self.assertTrue(' ' + PostEvaluationTask.EVALUATIONDIR_ARG +
                            ' ' + etask.get_dir() in res)
            self.assertTrue(' ' + PostEvaluationTask.EVALUATIONDIR_ARG +
                            ' ' + etask2.get_dir() in res)
            self.assertTrue(' ' + PostEvaluationTask.EVALUATIONDIR_ARG +
                            ' ' + etask3.get_dir() in res)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = PostEvaluationTask(temp_dir, params)
            task.create_dir()
            task.run()
            self.assertEqual(task.get_error(),
                             task.get_dir_name() +
                             ' already exists and status is ' +
                             D3RTask.UNKNOWN_STATUS)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_postevaluation_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            task = PostEvaluationTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'postevaluation not set')
            # test files get created
            self.assertEqual(os.path.isdir(task.get_dir()),
                             True)
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fail_no_evaluations_and_no_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.postevaluation = 'true'
            task = PostEvaluationTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'Did not find any Evaluation tasks to summarize')
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)
            self.assertTrue(os.path.isfile(os.path.join(task.get_dir(),
                                                        D3RTask.ERROR_FILE)))
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds_one_evaluation_and_no_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.postevaluation = 'echo'
            etask = EvaluationTask(temp_dir, 'foo_dock.' +
                                   EvaluationTaskFactory.SCORING_SUFFIX,
                                   None, params)
            etask.create_dir()

            task = PostEvaluationTask(temp_dir, params)
            ctask = ChallengeDataTask(temp_dir, params)
            task.run()

            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.COMPLETE_STATUS)
            cfile = os.path.join(task.get_dir(), D3RTask.COMPLETE_FILE)
            self.assertTrue(os.path.isfile(cfile))
            stdout_file = os.path.join(task.get_dir(), 'echo' +
                                       D3RTask.STDOUT_SUFFIX)
            self.assertTrue(os.path.isfile(stdout_file))

            f = open(stdout_file, 'r')
            data = f.read()
            f.close()
            ctaskdir = os.path.join(ctask.get_dir(),
                                    ctask.get_celpp_challenge_data_dir_name())

            self.assertTrue(' --evaluationdir ' + etask.get_dir() in data)
            self.assertTrue(' --challengedir ' + ctaskdir in data)

            self.assertTrue('--stageprefix stage.7. --evaluationsuffix ' +
                            '.extsubmission.evaluation$|.evaluation$' in data)

            stderr_file = os.path.join(task.get_dir(), 'echo' +
                                       D3RTask.STDERR_SUFFIX)
            self.assertTrue(os.path.isfile(stderr_file))
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_one_evaluation_and_no_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.postevaluation = 'false'
            etask = EvaluationTask(temp_dir, 'foo_dock.' +
                                   EvaluationTaskFactory.SCORING_SUFFIX,
                                   None, params)
            etask.create_dir()

            task = PostEvaluationTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'Non zero exit code: 1 received. '
                             'Standard out:  Standard error: ')
            self.assertEqual(task.get_status(), D3RTask.ERROR_STATUS)
            self.assertTrue(os.path.isfile(os.path.join(task.get_dir(),
                                                        D3RTask.ERROR_FILE)))
            self.assertTrue('Caught exception trying to '
                            'send evaluation email' in task.get_email_log())
            stdout_file = os.path.join(task.get_dir(), 'false' +
                                       D3RTask.STDOUT_SUFFIX)
            self.assertTrue(os.path.isfile(stdout_file))

            stderr_file = os.path.join(task.get_dir(), 'false' +
                                       D3RTask.STDERR_SUFFIX)
            self.assertTrue(os.path.isfile(stderr_file))
        finally:
            shutil.rmtree(temp_dir)

    def test_get_reply_to_address(self):
        pee = PostEvaluationEmailer('bob@bob.com', None)
        self.assertEqual(pee._get_reply_to_address(None), None)
        self.assertEqual(pee._get_reply_to_address('how@how.com'),
                         'how@how.com')

        pee = PostEvaluationEmailer('bob@bob.com', 'joe@joe.com')
        self.assertEqual(pee._get_reply_to_address(None), 'joe@joe.com')
        self.assertEqual(pee._get_reply_to_address('h@h.com'), 'joe@joe.com')

    def test_append_to_message_log(self):
        pee = PostEvaluationEmailer('bob@bob.com', 'joe@joe.com')
        self.assertEqual(pee.get_message_log(), None)
        pee._append_to_message_log('hello')
        self.assertEqual(pee.get_message_log(), 'hello')
        pee._append_to_message_log('bye')
        self.assertEqual(pee.get_message_log(), 'hellobye')

    def test_send_postevaluation_email_none_task_and_none_email(self):
        pee = PostEvaluationEmailer('bob@bob.com', 'joe@joe.com')
        pee.send_postevaluation_email(None)
        self.assertEqual(pee.get_message_log(), '\nTask passed in is None\n')

        pee = PostEvaluationEmailer(None, None)
        pee.send_postevaluation_email('foo')
        self.assertEqual(pee.get_message_log(),
                         '\nNo email addresses in to list\n')

    def test_generate_post_evaluation_email_body(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            weekyear = os.path.join(temp_dir, '2017', 'dataset.week.12')
            os.makedirs(weekyear, mode=0o755)
            task = PostEvaluationTask(weekyear, params)
            task.create_dir()
            f = open(task.get_summary_txt(), 'w')
            f.write('hello')
            f.flush()
            f.close()
            pee = PostEvaluationEmailer(None, None)
            subject, msg = pee._generate_post_evaluation_email_body(task)

            self.assertEqual(subject, D3RTask.SUBJECT_LINE_PREFIX +
                             '2017 week 12 post evaluation summary report')

            self.assertEqual(msg, 'Dear CELPP Admin,\n\nHere is the post '
                                  'evaluation  summary reports for CELPP '
                                  'week 12\n\n\n\nhello\n\n\nSincerely,\n\n'
                                  'CELPP Automation')
        finally:
            shutil.rmtree(temp_dir)

    def test_send_postevaluation_email_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            weekyear = os.path.join(temp_dir, '2017', 'dataset.week.12')
            os.makedirs(weekyear, mode=0o755)
            task = PostEvaluationTask(weekyear, params)
            task.create_dir()

            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock()
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            pee = PostEvaluationEmailer(['bob@bob.com'], 'joe@joe.com')
            pee.set_alternate_smtp_emailer(smtpemailer)
            task.set_evaluation_emailer(pee)
            f = open(task.get_summary_txt(), 'w')
            f.write('hello')
            f.flush()
            f.close()
            pee.send_postevaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(pee.get_message_log(),
                             '\nSent post evaluation email to: bob@bob.com\n')
            self.assertEqual(mockserver.sendmail.call_count, 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_send_postevaluation_email_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            weekyear = os.path.join(temp_dir, '2017', 'dataset.week.12')
            os.makedirs(weekyear, mode=0o755)
            task = PostEvaluationTask(weekyear, params)
            task.create_dir()
            csvfile = os.path.join(task.get_dir(), 'foo.csv')
            open(csvfile, 'a').close()

            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock(side_effect=IOError('ha'))
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            pee = PostEvaluationEmailer(['bob@bob.com'], 'joe@joe.com')
            pee.set_alternate_smtp_emailer(smtpemailer)
            task.set_evaluation_emailer(pee)
            f = open(task.get_summary_txt(), 'w')
            f.write('hello')
            f.flush()
            f.close()
            pee.send_postevaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(pee.get_message_log(),
                             '\nCaught exception trying to email :'
                             ' bob@bob.com : Caught exception ha\n')
            self.assertEqual(mockserver.sendmail.call_count, 1)
        finally:
            shutil.rmtree(temp_dir)

    # def test_run_succeeds_with_emailer(self):

    def test_run_succeeds_one_evaluation_with_emailer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()

            foo_script = os.path.join(temp_dir, 'foo.py')
            f = open(foo_script, 'w')
            f.write('#! /usr/bin/env python\n')
            f.write('import sys\nimport os\n')
            f.write('outdir = sys.argv[1]\n')
            f.write('sys.stdout.write(" ".join(sys.argv))\n')
            f.write('f = open(os.path.join(outdir, "summary.txt"),"w")\n')
            f.write('f.write("summary")\n')
            f.write('f.flush()\nf.close()\n')
            f.write('f = open(os.path.join(outdir, "yo.csv"), "a")\n')
            f.write('f.write("hi")\nf.flush()\nf.close()\n')
            f.write('sys.exit(0)\n')
            f.flush()
            f.close()
            os.chmod(foo_script, stat.S_IRWXU)

            params.postevaluation = foo_script
            etask = EvaluationTask(temp_dir, 'foo_dock.' +
                                   EvaluationTaskFactory.SCORING_SUFFIX,
                                   None, params)
            etask.create_dir()
            task = PostEvaluationTask(temp_dir, params)

            # setup emailer
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock()
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            pee = PostEvaluationEmailer(['bob@bob.com'], 'joe@joe.com')
            pee.set_alternate_smtp_emailer(smtpemailer)
            task.set_evaluation_emailer(pee)

            ctask = ChallengeDataTask(temp_dir, params)
            task.run()

            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_status(), D3RTask.COMPLETE_STATUS)
            cfile = os.path.join(task.get_dir(), D3RTask.COMPLETE_FILE)
            self.assertTrue(os.path.isfile(cfile))
            stdout_file = os.path.join(task.get_dir(), 'foo.py' +
                                       D3RTask.STDOUT_SUFFIX)
            self.assertTrue(os.path.isfile(stdout_file))

            f = open(stdout_file, 'r')
            data = f.read()
            f.close()
            ctaskdir = os.path.join(ctask.get_dir(),
                                    ctask.get_celpp_challenge_data_dir_name())

            self.assertTrue(' --evaluationdir ' + etask.get_dir() in data)
            self.assertTrue(' --challengedir ' + ctaskdir in data)

            self.assertTrue('--stageprefix stage.7. --evaluationsuffix ' +
                            '.extsubmission.evaluation$|.evaluation$' in data)

            stderr_file = os.path.join(task.get_dir(), 'foo.py' +
                                       D3RTask.STDERR_SUFFIX)

            f = open(stdout_file, 'r')
            data = f.read()
            f.close()
            self.assertTrue(' --evaluationdir ' + etask.get_dir() in data)
            self.assertTrue(os.path.isfile(stderr_file))
            self.assertEqual(task.get_error(), None)
            self.assertTrue('Sent post evaluation email to: '
                            'bob@bob.com' in task.get_email_log())
        finally:
            shutil.rmtree(temp_dir)

    """
    def test_generate_external_submission_email_body(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            dtask = D3RTask('/foo', params)
            dtask.set_name('foo')
            task = EvaluationTask(temp_dir, dtask.get_name(), dtask, params)
            task.create_dir()
            emailer = EvaluationEmailer(None, None)
            task.set_evaluation_emailer(emailer)
            rmsd = task.get_rmsd_txt()
            f = open(rmsd, 'w')
            f.write('  LMCSS  XXX\n1fcz  0.2  0.4\n')
            f.flush()
            f.close()
            subject, body = emailer\
                ._generate_external_submission_email_body(task)
            self.assertEqual(subject,
                             '[d3rcelpp] Week 0 evaluation results for foo')
            self.assertEqual(body, 'Dear CELPP Participant,\n\nHere are your '
                                   'docking evaluation results '
                                   '(RMSD, Angstroms) '
                                   'for CELPP week '
                                   '0\n\n\nEvaluation of docking'
                                   '\n=====================\n  LMCSS  XXX\n'
                                   '1fcz  0.2  0.4\n\n\n\nSincerely,\n\n'
                                   'CELPP Automation')
        finally:
            shutil.rmtree(temp_dir)

    def test_eval_emailer_append_to_message_log(self):
        emailer = EvaluationEmailer(None, None)
        self.assertEqual(emailer.get_message_log(), None)
        emailer._append_to_message_log('hi\n')
        self.assertEqual(emailer.get_message_log(), 'hi\n')
        emailer._append_to_message_log('how\n')
        self.assertEqual(emailer.get_message_log(), 'hi\nhow\n')

    def test_eval_emailer_get_external_submitter_email_none_from_guid(self):
        emailer = EvaluationEmailer(None, None)
        params = D3RParameters()
        task = EvaluationTask('/foo', None, None, params)
        self.assertEqual(emailer._get_external_submitter_email(task), None)
        self.assertEqual(emailer.get_message_log(),
                         '\nUnable to extract guid\n')


    def test_get_external_submitter_email_no_participant_email(self):

        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('444' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        plist = [Participant('1name', '1d3rusername', '444',
                             None)]
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        self.assertEqual(emailer._get_external_submitter_email(task), None)
        self.assertEqual(emailer.get_message_log(),
                         '\nEmail not set for participant\n')

    def test_get_external_submitter_email_valid(self):

        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        plist = [Participant('1name', '1d3rusername', '12345',
                             'bob@bob.com')]
        # try single email address
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        emails = emailer._get_external_submitter_email(task)
        self.assertEqual(emails[0], 'bob@bob.com')
        self.assertEqual(len(emails), 1)
        self.assertEqual(emailer.get_message_log(), None)

        # try multiple email address
        plist = [Participant('1name', '1d3rusername', '12345',
                             'bob@bob.com,joe@joe.com')]
        # try single email address
        emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
        emails = emailer._get_external_submitter_email(task)
        self.assertEqual(emails[0], 'bob@bob.com')
        self.assertEqual(emails[1], 'joe@joe.com')
        self.assertEqual(len(emails), 2)
        self.assertEqual(emailer.get_message_log(), None)

    def test_send_evaluation_email_none_task(self):
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(None)
        self.assertEqual(emailer.get_message_log(),
                         '\nTask passed in is None\n')

    def test_send_evaluation_email_clears_its_logs(self):
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(None)
        emailer.send_evaluation_email(None)

        self.assertEqual(emailer.get_message_log(),
                         '\nTask passed in is None\n')

    def test_send_evaluation_email_not_external_task(self):
        emailer = EvaluationEmailer(None, None)
        task = EvaluationTask('/foo', 'blah' +
                              EvaluationTask.EXT_SUBMISSION_SUFFIX,
                              None,
                              D3RParameters())
        emailer.send_evaluation_email(task)
        self.assertEqual(emailer.get_message_log(),
                         '\nNot an external submission\n')

    def test_send_evaluation_email_no_database(self):
        params = D3RParameters()
        dtask = D3RTask('/foo', params)
        dtask.set_name('444' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        task = EvaluationTask('/foo',
                              dtask.get_name(),
                              dtask, params)
        emailer = EvaluationEmailer(None, None)
        emailer.send_evaluation_email(task)
        self.assertEqual(emailer.get_message_log(),
                         '\nParticipant database is None cannot send docking '
                         'evaluation email!!!\n')

    def test_send_external_submission_email_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            task.create_dir()
            f = open(task.get_rmsd_txt(), 'w')
            f.write('hi\n')
            f.flush()
            f.close()
            plist = [Participant('1name', '1d3rusername', '12345',
                                 'bob@bob.com')]
            # try single email address
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock()
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)

            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(emailer.get_message_log(),
                             '\nSent evaluation email to: bob@bob.com\n')
            self.assertEqual(mockserver.sendmail.call_count, 1)

        finally:
            shutil.rmtree(temp_dir)

    def test_send_external_submission_email_sendmail_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            plist = [Participant('1name', '1d3rusername', '12345',
                                 'bob@bob.com')]
            # try single email address
            smtpemailer = SmtpEmailer()
            mockserver = D3RParameters()
            mockserver.sendmail = Mock(side_effect=IOError('ha'))
            mockserver.quit = Mock()
            smtpemailer.set_alternate_smtp_server(mockserver)
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            mockserver.quit.assert_any_call()
            self.assertEqual(emailer.get_message_log(),
                             '\nCaught exception trying to email '
                             'participant : Caught exception ha\n')

        finally:
            shutil.rmtree(temp_dir)

    def test_send_external_submission_email_no_submitter_email(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.program = 'foo'
            params.version = '1'
            dtask = D3RTask('/foo', params)
            dtask.set_name('12345' + EvaluationTask.EXT_SUBMISSION_SUFFIX)
            task = EvaluationTask(temp_dir,
                                  dtask.get_name(),
                                  dtask, params)
            plist = [Participant('1name', '1d3rusername', '1234',
                                 'bob@bob.com')]
            # try single email address
            emailer = EvaluationEmailer(ParticipantDatabase(plist), None)
            smtpemailer = SmtpEmailer()
            emailer.set_alternate_smtp_emailer(smtpemailer)
            emailer.send_evaluation_email(task)
            self.assertEqual(emailer.get_message_log(),
                             '\nNo participant found with guid: 12345\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_reply_to_address(self):
        # test get reply_to where no replytoaddress is set
        emailer = EvaluationEmailer(None, None)
        val = emailer._get_reply_to_address('bob@bob.com')
        self.assertEqual(val, 'bob@bob.com')

        emailer = EvaluationEmailer(None, 'joe@joe.com')
        val = emailer._get_reply_to_address('bob@bob.com')
        self.assertEqual(val, 'joe@joe.com')

    def test_get_smtp_emailer_valid(self):
        try:
            emailer = EvaluationEmailer(None, None)
            ss = emailer._get_smtp_emailer()
            self.assertTrue(ss is not None)
        finally:
            pass
    """

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'
__project__ = 'blastnfilter'

import os
import logging
import in_put
from d3r.blast.hit import Hit
from d3r.blast.ligand import Ligand
from d3r.filter.filter import QueryFilter
from d3r.filter.filter import HitFilter
from d3r.filter.filter import CandidateFilter
import out_put

logger = logging.getLogger(__name__)
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )


def split_input(options):
    """
    Split out and return command line options
    :param options:
    :return: non_polymer (string), polymer (string), out_dir (string), blast_dir (string), pdb_db (string), ->
    -> fasta (string)
    """
    non_polymer = os.path.abspath(options.non_polymer)
    polymer = os.path.abspath(options.polymer)
    ph = os.path.abspath(options.ph)
    out_dir = os.path.abspath(options.out)
    blast_dir = os.path.abspath(options.blast_db)
    pdb_path = os.path.abspath(options.pdb_path)
    pdb_db = os.path.join(blast_dir, 'pdb_db')
    fasta = os.path.join(blast_dir, 'pdb_seqres.txt')
    compinchi = os.path.abspath(options.compinchi)
    return non_polymer, polymer, ph, out_dir, blast_dir, pdb_db, pdb_path, fasta, compinchi


def blast_the_query(query, pdb_db, pdb_path, fasta, out_dir, compinchi):
    """
    Runs a blast search using the target sequence as a query
    :param query: an instance of blast.query.Query
    :param pdb_db: The absolute path to a BLASTP database
    :param pdb_path: The absolute path to a decompressed copy of the PDB
    :param fasta: A file with fasta sequences of each chain in the PDB
    :param out_dir: The path to the directory where results will be written
    :param compinchi: The absolute path to a file with InChI strings for each ligand in the PDB
    :return: query
    """
    if not Hit.pdb_dir:
        logger.debug('Hit.set_pdb_dir')
        Hit.set_pdb_dir(pdb_path)
    if not Hit.pdb_dict:
        logger.debug('Hit.set_pdb_dict')
        Hit.set_pdb_dict(fasta)
    if not Ligand.inchi_component:
        logger.debug('Ligand.set_inchi_component')
        Ligand.set_inchi_component(compinchi)
    logger.debug('query.run_blast')
    records = query.run_blast(pdb_db, out_dir)
    if records:
        logger.debug('query.set_hits')
        query.set_hits(records)
        logger.debug('query.fill_sequences')
        query.fill_sequences()
    return query


def calculate_mcss(query):
    """
    Calculates the maximum common substructure (MCSS) between each dockable-query ligand and each dockable-BLAST-hit
    ligand
    :param queries: a list of Query objects
    """
    if query.dock_count == 0:
        pass
    else:
        for hit in [hit for hit in query.hits if hit.dock_count > 0]:
            for hit_ligand in hit.dock:
                for query_ligand in query.dock:
                    mcss_mol = hit_ligand.mcss(query_ligand)
                    tanimoto_score = hit_ligand.calc_tanimoto(query_ligand)
                    if mcss_mol and tanimoto_score:
                        hit_ligand.set_mcss(query_ligand, mcss_mol, tanimoto_score)
            hit.set_maxmin_mcss()


def query_filter(query):
    """
    Runs query filtering operations
    :param query:
    :return:
    """
    filter = QueryFilter(query)
    filter.filter_apo()
    filter.filter_by_sequence_count()
    filter.filter_by_dockable_ligand_count()
    filter.filter_by_inchi_error()
    filter.filter_by_sequence_type()


def hit_filter(query):
    """
    Runs blast hit filtering operations
    :param query:
    :return:
    """
    h_filter = HitFilter(query)
    h_filter.filter_by_coverage()
    h_filter.filter_by_identity()
    h_filter.filter_by_sequence_count()
    h_filter.filter_by_dockable_ligand_count()
    h_filter.filter_apo()
    h_filter.filter_by_method()


def candidate_filter(queries):
    """
    Runs candidate filtering operations
    :param queries:
    :return:
    """
    c_filter = CandidateFilter(queries)
    c_filter.filter_holo()
    c_filter.filter_apo()
    c_filter.filter_for_most_similar()
    c_filter.filter_for_least_similar()
    c_filter.filter_for_highest_tanimoto()


#from memory_profiler import profile
#@profile(precision=4)
def run(options):
    """
    Run the BlastNFilter components
    :param options:
    """
    non_polymer, polymer, ph, out_dir, blast_dir, pdb_db, pdb_path, fasta, compinchi = split_input(options)
    logger.debug('Creating queries')
    queries = in_put.create_queries(polymer, non_polymer, ph)
    out_put.input_analysis(out_dir, queries)
    out_analysis = out_put.OutController()
    out_analysis.print_filter_criteria(out_dir)
    logger.debug("# queries " + str(len(queries)))
    while queries:
        #here the pop method extract the last item in the list and remove this item from the original list
        query = queries.pop()
        query_filter(query)
        if not query.triage:
            logger.debug('Blasting query:  ' + query.pdb_id)
            print "Blasting query:  %s "%query.pdb_id
            logging.info("Blasting query:  %s "%query.pdb_id)
            query = blast_the_query(query, pdb_db, pdb_path, fasta, out_dir, compinchi)
            logger.debug('calculate mcss')
            calculate_mcss(query)
            logger.debug('hit_filter')
            hit_filter(query)
            logger.debug('hit_filter')
            candidate_filter(query)
            logger.debug('set_query')
        out_analysis.set_query(query)
        out_put.writer(out_dir, query, True)
    out_analysis.print_to_file(out_dir)




# -*- coding: utf-8 -*-

__author__ = 'churas'

import os
import logging

from d3r.celpp.task import D3RTask
from d3r.celpp.chimeraprep import ChimeraProteinLigPrepTask

logger = logging.getLogger(__name__)


class AutoDockVinaTask(D3RTask):
    """Performs Auto Dock Vina docking

    """
    def __init__(self, path, args):
        super(AutoDockVinaTask, self).__init__(path, args)
        self.set_name('autodockvina')

        prep = ChimeraProteinLigPrepTask(path, args)
        self.set_stage(prep.get_stage() + 1)
        self.set_status(D3RTask.UNKNOWN_STATUS)

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain stderr/stdout files
           returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(AutoDockVinaTask, self).get_uploadable_files()
        return file_list

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the `ProteinLigPrep` task
           has `D3RTask.COMPLETE_STATUS` for
           status.  The method then verifies a `AutoDockVinaTask` does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check blast
        ligprep = ChimeraProteinLigPrepTask(self._path, self._args)
        ligprep.update_status_from_filesystem()
        if ligprep.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + 'task ' +
                        'because ' + ligprep.get_name() + 'task' +
                        'has a status of ' + ligprep.get_status())
            self.set_error(ligprep.get_name() + ' task has ' +
                           ligprep.get_status() + ' status')
            return False

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs AutoDockVinaTask after verifying chimeraproteinligprep was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes blastnfilter script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(AutoDockVinaTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('vina set to ' +
                         self.get_args().vina)
        except AttributeError:
            self.set_error('vina not set')
            self.end()
            return

        ligprep = ChimeraProteinLigPrepTask(self._path, self._args)

        #
        # vinadocking.py --structuredir <path to stage.3.chimeraprep> \
        # --outdir <path to stage.4.vina>
        #
        cmd_to_run = (self.get_args().vina + ' --structuredir ' +
                      ligprep.get_dir() +
                      ' --outdir ' + self.get_dir())

        vina_name = os.path.basename(self.get_args().vina)

        self.run_external_command(vina_name, cmd_to_run,
                                  True)
        # assess the result
        self.end()



__author__ = 'churas'

import os
import logging
import tarfile
import shutil

from d3r.celpp import util
from d3r.celpp.task import D3RTask
from d3r.celpp.blastnfilter import BlastNFilterTask
from d3r.celpp.dataimport import DataImportTask

logger = logging.getLogger(__name__)


class ChallengeDataTask(D3RTask):
    """Generates ChallengeData files

    """

    TAR_EXCLUDE_DIRS = ['error_container']
    TAR_EXCLUDE_FILES = ['final.log']
    TAR_GZ_SUFFIX = ".tar.gz"
    README_TXT_FILE = "readme.txt"
    LATEST_TXT = "latest.txt"

    README_BODY = """CELPP Weekly Pose Prediction Challenge
======================================

Celpprunner version: {version}
Week: {week}
Year: {year}

This tar file contains the CELPP weekly pose prediction challenge dataset.

Within this readme.txt is a description of the data in this tar file as well as
a summary of the Blastnfilter run which generated these candidates.

Tsv files downloaded from
=========================

http://www.wwpdb.org/files/new_release_structure_sequence_canonical.tsv
http://www.wwpdb.org/files/new_release_structure_nonpolymer.tsv
http://www.wwpdb.org/files/new_release_crystallization_pH.tsv

(If new_release_structure_sequence_canonical.tsv is NOT available then
 http://www.wwpdb.org/files/new_release_structure_sequence.tsv will be
 downloaded instead)

Structure of data overview
==========================

This tar file contains a set of directories set to the name of Targets. Targets
are proteins which have primary sequence released, but not 3D coordinates.

Within each directory are a set of Candidates.  Candiates are proteins with
similar structure to the Target that also have known 3D coordinates which can
be used for pose prediction.

For more information visit:

https://github.com/drugdata/D3R
              or
https://drugdesigndata.org/about/celpp


Structure of data
=================

Below is a definition of the files and directories within this tar file:

[file or directory <text within denote values that change>]

  -- Definition


 [readme.txt]

     -- Description of data and output from celpp blastnfilter stage of
        processing.

 [new_release_crystallization_pH.tsv]
 [new_release_structure_nonpolymer.tsv]
 [new_release_structure_sequence_canonical.tsv]

     -- Tsv files downloaded from: http://www.wwpdb.org/files
        (If new_release_structure_sequence_canonical.tsv is NOT available
         new_release_structure_sequence.tsv will be used instead)

 [<target id>]/
               [<target id>.txt]

                  -- Summary of Blastnfilter results for target protein
                     with PDBID.

               [LMCSS-<target id>_<candidate id>-<candidate ligand id>.pdb]

                  -- Candidate protein for docking which:
                      1) Passes the Blastnfilter criteria

                      2) Contains the Ligand with the largest maximum common
                         substructure (MCSS) to the Target Ligand.

                         Note:  If multiple proteins founded, the protein
                                with the highest resolution will be picked.

               [SMCSS-<target id>_<candidate id>-<candidate ligand id>.pdb]

                  -- Candidate protein for docking which:
                      1) Passes the Blastnfilter criteria.

                      2) Contains the Ligand with the smallest maximum common
                         substructure (MCSS) to the Target Ligand.

                         Note:  If multiple proteins founded, the protein
                                with the highest resolution will be picked.

               [hiResHolo-<target id>_<candidate id>-<candidate ligand id>.pdb]

                   -- Candidate protein for docking which:
                      1) Passes the Blastnfilter criteria.

                      2) Has the highest resolution among all holo proteins.

               [hiResApo-<target id>_<candidate id>-<candidate ligand id>.pdb]

                   -- Candidate protein for docking which:
                      1) Passes the Blastnfilter criteria.

                      2) Has the highest resolution among all apo proteins.

               [hiTanimoto-<target id>_<candidate id>-<candidate ligand id>\
                                                                          .pdb]

                   -- Candidate protein for docking which:
                      1) Passes the Blastnfilter criteria.

                      2) Contains the Ligand with the highest structural
                         similarity in Tanimoto score to the Target Ligand.

                         Note:  If multiple proteins founded, the protein
                                with the highest resolution will be picked.

               [LMCSS-<target id>_<candidate id>-<candidate ligand id> \
                                                                      -lig.pdb]

                    -- Contains the 3D coordinate of the atoms for the ligand
                       in the MaxMCSS candidate (LMCSS) protein.

               [lig_<candidate ligand id>.smi]

                   -- Canonical smile string of the Target Ligand which will be
                      used in later docking.

               [lig_<candidate ligand id>.inchi]

                   -- Inchi string of the Target Ligand.

               [lig_<candidate ligand id>.mol]

                   -- 2D structure of the Target Ligand.

Blastnfilter Summary
====================

"""

    def __init__(self, path, args):
        super(ChallengeDataTask, self).__init__(path, args)
        self.set_name('challengedata')

        # Make stage number one higher then BlastNFilter Stage
        blast = BlastNFilterTask(path, args)
        self.set_stage(blast.get_stage() + 1)

        self.set_status(D3RTask.UNKNOWN_STATUS)
        self._challenge_tarball_filename = None
        self._week_num = util.get_celpp_week_number_from_path(self.get_path())
        self._year = util.get_celpp_year_from_path(self.get_dir())

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files in addition to stderr/stdout files
           if they are found on the filesystem


           :returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(ChallengeDataTask, self).get_uploadable_files()
        tarfile = self.get_celpp_challenge_data_tar_file()
        if os.path.isfile(tarfile):
            file_list.append(tarfile)
        else:
            logger.warning('No tar file found!!!')

        return file_list

    def get_celpp_challenge_data_tar_file(self):
        """Returns path to challenge tar ball
        """
        return os.path.join(self.get_dir(),
                            self.get_celpp_challenge_data_dir_name() +
                            ".tar.gz")

    def get_celpp_challenge_data_dir_name(self):
        """Returns path to celpp challenge data directory name
        """
        return 'celpp_week' + str(self._week_num) + '_' + str(self._year)

    def _create_challenge_dir(self):
        """Creates the challenge directory
        """
        challenge_dir = os.path.join(self.get_dir(),
                                     self.get_celpp_challenge_data_dir_name())

        logger.debug('Creating directory: ' + challenge_dir)
        if not os.path.isdir(challenge_dir):
            os.mkdir(challenge_dir)
        return challenge_dir

    def _create_readme(self, path):
        """Creates readme.txt file for task

        """
        ver = 'Unknown'
        try:
            ver = self.get_args().version
        except AttributeError:
            logger.warning('Version unset using Unknown')

        f = open(os.path.join(path, ChallengeDataTask.README_TXT_FILE), 'w')
        f.write(ChallengeDataTask.README_BODY.format(
            version=ver,
            week=self._week_num,
            year=self._year))

        blast = BlastNFilterTask(self.get_path(), self.get_args())
        summary_file = blast.get_blastnfilter_summary_file()

        # append summary.txt file
        if os.path.isfile(summary_file):
            sumfile = open(summary_file, 'r')
            for line in sumfile:
                f.write(line)
            sumfile.close()

        f.flush()
        f.close()

    def _copy_over_tsv_files(self, challenge_dir):
        """Copies over tsv files from `DataImportTask`
        """
        dataimport = DataImportTask(self.get_path(), self.get_args())

        crystal_dest = os.path.join(challenge_dir,
                                    DataImportTask.CRYSTALPH_TSV)

        if os.path.isfile(dataimport.get_crystalph_tsv()):
            logger.debug('Copying over ' + dataimport.get_crystalph_tsv() +
                         'to ' + crystal_dest)
            shutil.copyfile(dataimport.get_crystalph_tsv(), crystal_dest)
        else:
            logger.warning(dataimport.get_crystalph_tsv() +
                           ' file does not exist')

        nonpoly_dest = os.path.join(challenge_dir,
                                    DataImportTask.NONPOLYMER_TSV)

        if os.path.isfile(dataimport.get_nonpolymer_tsv()):
            logger.debug('Copying over ' + dataimport.get_nonpolymer_tsv() +
                         'to ' + nonpoly_dest)
            shutil.copyfile(dataimport.get_nonpolymer_tsv(), nonpoly_dest)
        else:
            logger.warning(dataimport.get_nonpolymer_tsv() +
                           ' file does not exist')

        seq_dest = os.path.join(challenge_dir,
                                DataImportTask.SEQUENCE_TSV)

        if os.path.isfile(dataimport.get_sequence_tsv()):
            logger.debug('Copying over ' + dataimport.get_sequence_tsv() +
                         'to ' + seq_dest)
            shutil.copyfile(dataimport.get_sequence_tsv(), seq_dest)
        else:
            logger.warning(dataimport.get_sequence_tsv() +
                           ' file does not exist')

    def _tar_challenge_dir(self, challenge_dir_name):
        """Creates compressed tarball of challenge directory
        """

        tfile = os.path.join(self.get_dir(),
                             challenge_dir_name +
                             ChallengeDataTask.TAR_GZ_SUFFIX)

        logger.debug('Creating tar file: ' + tfile)

        tar = tarfile.open(tfile, 'w:gz')
        challenge_dir = os.path.join(self.get_dir(), challenge_dir_name)

        for entry in os.listdir(challenge_dir):
            fullpath = os.path.join(challenge_dir, entry)
            if os.path.isdir(fullpath):
                if entry in ChallengeDataTask.TAR_EXCLUDE_DIRS:
                    logger.debug('Skipping insertion of ' + entry +
                                 ' into tar file')
                    continue

                logger.debug('Adding directory to tarball: ' + entry)
                tar.add(fullpath, arcname=challenge_dir_name + '/' + entry)
                continue
            if os.path.isfile(fullpath):
                if entry in ChallengeDataTask.TAR_EXCLUDE_FILES:
                    logger.debug('Skipping insertion of ' + entry +
                                 ' into tar file')
                    continue
                logger.debug('Adding file to tarball: ' + entry)
                tar.add(fullpath, challenge_dir_name + '/' + entry)

        tar.close()
        tfile_size = str(os.path.getsize(tfile))
        logger.debug('Tarfile created and is: ' +
                     tfile_size + 'bytes in size')
        self.append_to_email_log('Tarfile: ' + tfile + ' (' +
                                 tfile_size + ' bytes) created.')
        return tfile

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the `BlastNFilterTask` task
           has `D3RTask.COMPLETE_STATUS` for
           status.  The method then verifies a `ProteinLigPrepTask` does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check blast
        blastnfilter = BlastNFilterTask(self._path, self._args)
        blastnfilter.update_status_from_filesystem()
        if blastnfilter.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + 'task ' +
                        'because ' + blastnfilter.get_name() + 'task' +
                        'has a status of ' + blastnfilter.get_status())
            self.set_error(blastnfilter.get_name() + ' task has ' +
                           blastnfilter.get_status() + ' status')
            return False

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def _upload_challenge_file(self, challenge_file):
        """Uploads challenge file to remote ftp server
        """
        if challenge_file is None:
            logger.error('challenge_file is None')
            self.append_to_email_log('challenge_file is None in '
                                     '_upload_challenge_file\n')
            return

        uploader = self.get_file_transfer()
        if uploader is None:
            logger.warning('No uploader available to upload challenge data')
            self.append_to_email_log('No uploader available to upload '
                                     'challenge data\n')
            return
        try:
            remote_dir = uploader.get_remote_challenge_dir()
            if remote_dir is None:
                logger.warning('No remote challenge directory set for '
                               'ftp upload')
                self.append_to_email_log('No remote challenge directory set ' +
                                         'for ftp upload\n')
                return
            logger.debug('Attempting to upload ' + challenge_file + ' to '
                         + remote_dir)

            logger.debug('Connecting to remote server to upload challenge data'
                         'package')
            # connect to remote server
            uploader.connect()

            if uploader.upload_file_direct(challenge_file, remote_dir,
                                           os.path.basename(challenge_file)) \
                    is False:
                raise Exception(uploader.get_error_msg())

            self.append_to_email_log('\nChallenge tarball uploaded: \n'
                                     + uploader.get_upload_summary() + '\n')

            # create latest.txt file and upload to ftp server
            self._upload_latest_file(uploader, challenge_file, remote_dir)
        finally:
            uploader.disconnect()

    def _upload_latest_file(self, uploader, challenge_file, remote_dir):
        """Creates and uploads latest.txt file containing name of tarfile
        """
        chall_name = os.path.basename(challenge_file)
        latest_file = os.path.join(self.get_dir(),
                                   ChallengeDataTask.LATEST_TXT)
        f = open(latest_file, 'w')
        f.write(chall_name)
        f.flush()
        f.close()
        if uploader.upload_file_direct(latest_file, remote_dir,
                                       ChallengeDataTask.LATEST_TXT) is False:
            raise Exception(uploader.get_error_msg())

    def run(self):
        """Runs ChallengeDataTask after verifying blastnfilter was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes blastnfilter script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(ChallengeDataTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('genchallenge set to ' +
                         self.get_args().genchallenge)
        except AttributeError:
            self.set_error('genchallenge not set')
            self.end()
            return

        try:
            logger.debug('pdbdb set to ' +
                         self.get_args().pdbdb)
        except AttributeError:
            self.set_error('pdbdb not set')
            self.end()
            return

        # create challenge dir
        challenge_dir = ''
        try:
            challenge_dir = self._create_challenge_dir()
            logger.debug('Made challenge dir ' + challenge_dir)
        except OSError:
                logger.exception('Problem making challenge dir ' +
                                 challenge_dir)
                self.set_error('Unable to create ' + challenge_dir)
                self.end()

        blastnfilter = BlastNFilterTask(self.get_path(),
                                        self.get_args())
        #
        # genchallengedata.py --candidatedir <path to stage.3.blastnfilter> \
        # --pdbdb <path to pdb database> \
        # --outdir <path to stage.4.challengedata/celpp_week#_####/>
        cmd_to_run = (self.get_args().genchallenge + ' --candidatedir ' +
                      blastnfilter.get_dir() +
                      ' --pdbdb ' + self.get_args().pdbdb +
                      ' --outdir ' + challenge_dir)

        genchallenge_name = os.path.basename(self.get_args().genchallenge)
        self.run_external_command(genchallenge_name, cmd_to_run,
                                  True)

        try:
            # create readme.txt file
            self._create_readme(challenge_dir)

            # copy over tsv files
            self._copy_over_tsv_files(challenge_dir)

            # tar and compress up celpp_week#_#### directory omitting final.log
            tfile = self._tar_challenge_dir(os.path.basename(challenge_dir))

            # upload tar file to remote server
            self._upload_challenge_file(tfile)

        except Exception as e:
            logger.exception('Caught exception')
            self.set_error('Caught exception ' + str(e))
        # assess the result
        logger.debug('About to complete challengedata.py')
        self.end()
        logger.debug('Completed challengedata.py')



__author__ = 'churas'

import unittest
import tempfile
import os.path

from d3r.blast.query import Query
"""
test_in_put
--------------------------------

Tests for `in_put` module.
"""

import shutil
from d3r.utilities import in_put


class TestInPut(unittest.TestCase):
    def setUp(self):
        pass

    def get_valid_new_release_sequence_tsv(self):
        val = """PDB_ID  Sequence_Count  Sequence
2N1I    1       GSGFPTSEDFTPKEGSPYEAPVYIPEDIPIPADFELRESSIPGAGLGVWAKRKMEAG
2N27    1       ADQLTEEQIAEFKEAFSLFDKDGDGTITTKELGTVMRSLGQNPTEAELQDMINEVDADEMIR
2N3M    1       (DT)(DG)(DG)(DT)(DG)(DG)(DT)(DG)(DG)(DT)(DT)(DG)(DT)(DT)(DG)
2N4M    1       (DG)(DT)(DG)(DC)(4E9)(DT)(DG)(DT)(DT)(DT)(DG)(DT)
2N4M    2       (DA)(DC)(DA)(DA)(DA)(DC)(DA)(DC)(DG)(DC)(DA)(DC)
2N7A    1       MEGDRQYGDGYLLQVQELVTVQEGLSVHVPCSFSYPQDGWTDSDPVHGYWFRAGDRPYQDAP
2N7B    1       MEGDRQYGDGYLLQVQELVTVQEGLSVHVPCSFSYPQDGWTDSDPVHGYWFRAGDRPYQDAP
2N9T    1       YCQKWMWTCDSERKCCEGMVCRLWCKKKLW"""
        return val

    def test_read_sequences_valid_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            tsvdata = self.get_valid_new_release_sequence_tsv()
            poly = os.path.join(temp_dir, 'foo.tsv')
            f = open(poly, 'w')
            f.write(tsvdata)
            f.flush()
            f.close()
            queries = in_put.read_sequences(poly)
            self.assertEqual(len(queries), 7)
            self.assertEqual(queries[0].pdb_id, '2n1i')
            self.assertEqual(queries[6].pdb_id, '2n9t')
        finally:
            shutil.rmtree(temp_dir)

    def test_read_sequences_invalid_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            poly = os.path.join(temp_dir, 'foo.tsv')
            f = open(poly, 'w')
            f.write('blah\nblah\n')
            f.flush()
            f.close()
            queries = in_put.read_sequences(poly)
            self.assertEqual(len(queries), 0)
        finally:
            shutil.rmtree(temp_dir)

    def get_valid_nonpolymer_tsv(self):
        val = """PDB_ID  Component_ID    InChI
2N1I    CA      InChI=1S/Ca/q+2
2N27    4DY     InChI=1S/C18H27NO3/c1-14(2)8-6-4-5-7-9-18(21)19-13-15-10
2N7B    0D8     InChI=1S/C3H9NO/c4-2-1-3-5/h5H,1-4H2
4XGS    OFO     InChI=1S/2Fe.H2O.O/h;;1H2;/q;+1;;/p-1
4YDE    EDO     InChI=1S/C2H6O2/c3-1-2-4/h3-4H,1-2H2
4YDE    ZN      InChI=1S/Zn/q+2"""
        return val

    def test_read_ligands_valid_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            tsvdata = self.get_valid_nonpolymer_tsv()
            nonpoly = os.path.join(temp_dir, 'foo2.tsv')
            f = open(nonpoly, 'w')
            f.write(tsvdata)
            f.flush()
            f.close()
            queries = []
            q = Query()
            q.pdb_id = '2n1i'
            queries.append(q)
            q = Query()
            q.pdb_id = '2n27'
            queries.append(q)
            q = Query()
            q.pdb_id = '2n7b'
            queries.append(q)
            in_put.read_ligands(nonpoly, queries)

        finally:
            shutil.rmtree(temp_dir)

    def get_valid_ph(self):
        val = """PDB_ID  _exptl_crystal_grow.pH
2N1I    6.5
2N27    8
2N7B    8
2N3M    5.5
2N4M    8.5
2N7A    8"""
        return val

    def test_read_ph_valid_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            tsvdata = self.get_valid_ph()
            ph = os.path.join(temp_dir, 'foo.tsv')
            f = open(ph, 'w')
            f.write(tsvdata)
            f.flush()
            f.close()
            queries = []
            q = Query()
            q.pdb_id = '2n1i'
            queries.append(q)
            q = Query()
            q.pdb_id = '2n27'
            queries.append(q)
            q = Query()
            q.pdb_id = '2n7b'
            queries.append(q)

            in_put.read_ph(ph, queries)

            self.assertEqual(queries[0].exp_ph, '6.5')
            self.assertEqual(queries[1].exp_ph, '8')
            self.assertEqual(queries[2].exp_ph, '8')

        finally:
            shutil.rmtree(temp_dir)

    def test_create_queries(self):
        temp_dir = tempfile.mkdtemp()
        try:
            poly = self.get_valid_new_release_sequence_tsv()
            poly_file = os.path.join(temp_dir, 'poly.tsv')
            f = open(poly_file, 'w')
            f.write(poly)
            f.flush()
            f.close()

            nopoly = self.get_valid_nonpolymer_tsv()
            nopoly_file = os.path.join(temp_dir, 'nopoly.tsv')
            f = open(nopoly_file, 'w')
            f.write(nopoly)
            f.flush()
            f.close()

            ph = self.get_valid_ph()
            ph_file = os.path.join(temp_dir, 'ph.tsv')
            f = open(ph_file, 'w')
            f.write(ph)
            f.flush()
            f.close()

            queries = in_put.create_queries(poly_file,
                                            nopoly_file, ph_file)
            self.assertEqual(len(queries), 7)
            self.assertEqual(queries[0].pdb_id, '2n1i')
            self.assertEqual(queries[6].pdb_id, '2n9t')

        finally:
            shutil.rmtree(temp_dir)

    def test_label(self):
        self.assertEqual(in_put.label('VA3'), 'do_not_call')
        self.assertEqual(in_put.label('hahaha'), 'dock')

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



__author__ = 'churas'

import os
import logging

from d3r.celpp.task import D3RTask
from d3r.celpp.challengedata import ChallengeDataTask

logger = logging.getLogger(__name__)


class ProteinLigPrepTask(D3RTask):
    """Performs preparation of PDB and inchi files

    """

    MAE_FILES = ['ligand.mae', 'LMCSS.maegz', 'SMCSS.maegz',
                 'hiResApo.maegz', 'hiResHolo.maegz']

    FINAL_LOG = 'final.log'
    TASK_NAME = 'proteinligprep'

    def __init__(self, path, args):
        super(ProteinLigPrepTask, self).__init__(path, args)
        self.set_name(ProteinLigPrepTask.TASK_NAME)

        # Make stage number one higher then BlastNFilter Stage
        chall = ChallengeDataTask(path, args)
        self.set_stage(chall.get_stage() + 1)

        self.set_status(D3RTask.UNKNOWN_STATUS)

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files in addition to stderr/stdout files
           if they are found on the filesystem

           final.log
           <pbid>/ligand.mae
           <pdbid>/LMCSS.maegz
           <pdbid>/SMCSS.maegz
           <pdbid>/hiResApo.maegz
           <pdbid>/hiResHolo.maegz

           :returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(ProteinLigPrepTask, self).get_uploadable_files()

        out_dir = self.get_dir()

        try:
            final_log = os.path.join(out_dir, ProteinLigPrepTask.FINAL_LOG)
            if os.path.isfile(final_log):
                file_list.append(final_log)

            for entry in os.listdir(out_dir):
                full_path = os.path.join(out_dir, entry)
                if not os.path.isdir(full_path):
                    continue
                logger.debug('Looking for .maegz files in ' + full_path)
                for mae_name in ProteinLigPrepTask.MAE_FILES:
                    mae = os.path.join(full_path, mae_name)
                    if os.path.isfile(mae):
                        file_list.append(mae)
        except OSError:
            logger.exception('Caught exception looking for pbdid folders')
        return file_list

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the `BlastNFilterTask` task
           has `D3RTask.COMPLETE_STATUS` for
           status.  The method then verifies a `ProteinLigPrepTask` does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check blast
        chall = ChallengeDataTask(self._path, self._args)
        chall.update_status_from_filesystem()
        if chall.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + 'task ' +
                        'because ' + chall.get_name() + 'task' +
                        'has a status of ' + chall.get_status())
            self.set_error(chall.get_name() + ' task has ' +
                           chall.get_status() + ' status')
            return False

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs ProteinLigPrepTask after verifying ChallengeDataTask was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes blastnfilter script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(ProteinLigPrepTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('proteinligprep set to ' +
                         self.get_args().proteinligprep)
        except AttributeError:
            self.set_error('proteinligprep not set')
            self.end()
            return

        try:
            logger.debug('pdbdb set to ' +
                         self.get_args().pdbdb)
        except AttributeError:
            self.set_error('pdbdb not set')
            self.end()
            return

        chall = ChallengeDataTask(self._path, self._args)

        #
        # proteinligprep.py --candidatedir <path to stage.3.challengedata> \
        # --outdir <path to stage.3.proteinligprep>

        challdir = os.path.join(chall.get_dir(),
                                chall.get_celpp_challenge_data_dir_name())
        cmd_to_run = (self.get_args().proteinligprep + ' --candidatedir ' +
                      challdir +
                      ' --pdbdb ' + self.get_args().pdbdb +
                      ' --outdir ' + self.get_dir())

        proteinligprep_name = os.path.basename(self.get_args().proteinligprep)

        self.run_external_command(proteinligprep_name, cmd_to_run,
                                  True)
        # assess the result
        self.end()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import tarfile
import shutil
import time

from d3r.celpp.task import D3RTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.evaluation import EvaluationTaskFactory
from d3r.celpp.evaluation import EvaluationTask
from d3r.celpp.filetransfer import FtpFileTransfer

logger = logging.getLogger(__name__)


class ChallengePackageDownloadError(Exception):
    """Raised when there was an error downloading
       Challenge data package
    """
    pass


class ExternalDataSubmissionFactory(object):
    """Factory to create ExternalDataSubmissionObjects
    """

    DOCKEDRESULTS = '_dockedresults_'

    def __init__(self, path, args):
        """Constructor
        """
        try:
            logger.debug('ftpconfig set to ' + args.ftpconfig)
            self._file_transfer = FtpFileTransfer(args.ftpconfig)
        except Exception:
            logger.exception('Caught exception')
            self._file_transfer = None
        self._path = path
        self._args = args
        ctask = ChallengeDataTask(path, args)
        self._chall_dir_name = ctask.get_celpp_challenge_data_dir_name()

    def get_args(self):
        return self._args

    def get_path(self):
        return self._path

    def _get_challenge_package_results_file_name(self, dir_name):
        return (self._chall_dir_name +
                ExternalDataSubmissionFactory.DOCKEDRESULTS +
                dir_name + ChallengeDataTask.TAR_GZ_SUFFIX)

    def set_file_transfer(self, filetransfer):
        """Sets file transfer
        """
        self._file_transfer = filetransfer

    def get_file_transfer(self):
        """Gets file transfer
        """
        return self._file_transfer

    def _get_submission_dirs(self, remote_dir):
        """Gets list of directories under `remote_dir`
        :param remote_dir: path on remote server to examine
        :returns: list of directory names without path prefix
        :raises AttributeError: if `get_file_transfer()` is None
        """
        dlist = self._file_transfer.list_dirs(remote_dir)
        if dlist is None:
            logger.debug('No directories returned')
            return []
        logger.debug('Found ' + str(len(dlist)) + ' directories on ' +
                     remote_dir)
        return dlist

    def _get_challenge_data_package_file(self, remote_dir, dir_name):
        """Gets challenge data package file under `remote_dir` / `dir_name`
           if it exists
           :returns: Path to remote challenge file upon success or None if
           not found
           :raises AttributeError if `get_file_transfer()` is None
        """
        ft = self.get_file_transfer()
        flist = ft.list_files(os.path.normpath(os.path.join(remote_dir,
                                                            dir_name)))
        if flist is None:
            logger.info('No files found in ' + dir_name)
            return None
        logger.debug('Found ' + str(len(flist)) + ' files in ' + dir_name)
        chall_fname = self._get_challenge_package_results_file_name(dir_name)
        logger.debug('Looking for ' + chall_fname + ' file in directory')

        for entry in flist:
            if entry == chall_fname:
                logger.info('Found matching entry ' + entry)
                return os.path.normpath(os.path.join(remote_dir,
                                                     dir_name,
                                                     chall_fname))
            else:
                logger.debug('Encountered non challenge file: ' + entry)

        return None

    def _remove_latest_txt(self):
        """Removes the latest.txt file from ftp server if found
        """
        try:
            ft = self.get_file_transfer()
            latest_txt = os.path.join(ft.get_remote_challenge_dir(),
                                      ChallengeDataTask.LATEST_TXT)
            logger.info('Attempting to remove ' + latest_txt)
            val = ft.delete_file(latest_txt)
            logger.info('Return value from delete call ' + str(val))
        except Exception:
            logger.exception('Caught exception trying to remove latest.txt')

    def get_external_data_submissions(self):
        """Generate ExternalDataSubmission objects
           Method should examine the submission directory on the
           remote server via FtpFileTransfer(args.ftpconfig)
           for each directory under submission directory
           look for a celpp_weekXX_YYYY_dockedresults_ZZZZ.tar.gz file
           where ZZZZ matches name of directory under submission directory
           Regardless if its found or not create new ExternalDataSubmission
           object and pass ZZZZ for name and path to .tar.gz file as
           remotefile if the tar.gz file was found otherwise pass None
           along with args.  Append this object to list and
           return it
        """
        task_list = []
        try:
            self._file_transfer.connect()
            self._remove_latest_txt()
            subdir = self._file_transfer.get_remote_submission_dir()
            dlist = self._get_submission_dirs(subdir)
            for d in dlist:
                chall_file = self._get_challenge_data_package_file(subdir, d)

                if chall_file is not None:
                    et = ExternalDataSubmissionTask(self.get_path(), d,
                                                    chall_file,
                                                    self.get_args())
                    logger.info('Added ExternalData Submission Task: ' +
                                et.get_dir_name())
                    task_list.append(et)
        except Exception:
            logger.exception('Caught exception')
        finally:
            try:
                self._file_transfer.disconnect()
            except Exception:
                logger.exception('Caught exception disconnecting')

        return task_list


class ExternalDataSubmissionTask(D3RTask):
    """Downloads external user docking Submissions
    """

    def __init__(self, path, name, remotefile, args):
        super(ExternalDataSubmissionTask, self).__init__(path, args)
        self.set_name(name + EvaluationTask.EXT_SUBMISSION_SUFFIX)
        self.set_stage(EvaluationTaskFactory.DOCKSTAGE)
        self.set_status(D3RTask.UNKNOWN_STATUS)
        self.set_remote_challenge_data_package(remotefile)
        self._maxretries = 3
        self._retrysleep = 30

    def set_download_max_retry_count(self, numretries):
        """Sets number of retries to perform before giving up on
           download
           :param numretries: int representing number of retries
        """
        self._maxretries = int(numretries)

    def set_download_retry_sleep(self, retrysleep):
        """Sets number of seconds to wait before retrying download
           :param retrysleep: int containing number of seconds
        """
        self._retrysleep = int(retrysleep)

    def set_remote_challenge_data_package(self, remotefile):
        self._remote_challenge_file = remotefile

    def get_remote_challenge_data_package(self):
        return self._remote_challenge_file

    def can_run(self):
        """Determines if task can actually run
           by seeing if this task has already been run
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None

        # check this task is not complete and does not exist
        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def _is_tarmembername_safe(self, tname, chall_name):
        if tname.startswith(os.sep):
            msg = ('Skipping, path starts with / : ' +
                   tname)
            logger.warning(msg)
            self.append_to_email_log(msg + '\n')
            return False

        if not tname.startswith(chall_name):
            msg = ('Skipping, path does not conform: ' + tname)
            logger.warning(msg)
            self.append_to_email_log(msg + '\n')
            return False

        if tname.find('..') > -1:
            msg = ('Skipping, found .. in path: ' + tname)
            logger.warning(msg)
            self.append_to_email_log(msg + '\n')
            return False
        return True

    def _untar_challenge_data_package(self, chall_file):
        """Untars tar file into task working directory
        """
        try:
            tar = tarfile.open(os.path.join(self.get_dir(), chall_file))
            chall_name = chall_file.replace(ChallengeDataTask.TAR_GZ_SUFFIX,
                                            '')
            for t in tar.getmembers():
                if self._is_tarmembername_safe(t.name, chall_name) is False:
                    continue

                prefix_dir_removed = t.name.replace(chall_name + '/', '', 1)
                logger.debug('Found entry in tar ' + t.name)
                if t.isdir():
                    logger.debug('Found directory ' + prefix_dir_removed)
                    continue
                if t.isfile():
                    logger.debug('Extracting file ' + prefix_dir_removed)
                    tar.extract(t, path=self.get_dir())
                    continue
                msg = ('Ignoring non dir/file entry in tar ' +
                       prefix_dir_removed)
                logger.warning(msg)
                self.append_to_email_log(msg + '\n')
            return chall_name
        finally:
            try:
                tar.close()
            except Exception:
                logger.exception('Caught exception trying to close tar')

    def _move_challenge_data_package_into_task_dir(self, chall_name):
        """Moves the contents of uncompressed challenge data package into
           `get_dir()` of task
        """
        package_dir = os.path.join(self.get_dir(), chall_name)
        for entry in os.listdir(package_dir):
            fullpath = os.path.join(package_dir, entry)
            logger.debug('Moving ' + fullpath)
            shutil.move(fullpath, os.path.join(self.get_dir(), entry))
        os.rmdir(package_dir)

    def _get_summary_of_docked_results(self):
        """Right now just returns an empty string
        """
        return ''

    def _download_remote_challenge_data_package_with_retry(self):
        """With retries attempts to download remote challenge data
           package
           :returns: string containing path to remote challenge data
                     package file or None upon failure.
        """
        chall_name = os.path.basename(self.get_remote_challenge_data_package())
        count = 0
        while count <= self._maxretries:
            logger.debug('Try # ' + str(count) + ' of ' +
                         str(self._maxretries) + ' to download ' +
                         chall_name)
            if count > 0:
                self.append_to_email_log('Try # ' + str(count) + ' of ' +
                                         str(self._maxretries) +
                                         ' to download ' + chall_name + '\n')

            try:
                res = self._download_remote_challenge_data_package(chall_name)
                if res is True:
                    logger.debug('Successfully downloaded')
                    return chall_name
                logger.debug('Download failed, sleeping ' +
                             str(self._retrysleep) + ' seconds')
                time.sleep(self._retrysleep)
            except Exception:
                logger.exception('Caught exception, retrying download')
            finally:
                count += 1

        raise ChallengePackageDownloadError('Unable to download ' + chall_name)

    def _download_remote_challenge_data_package(self, chall_name):
        """Downloads external submissions
           pseudo code:
           ft.download_file(self.get_remote_challenge_data_package())
        """
        try:

            ft = self.get_file_transfer()
            ft.connect()
            localfile = os.path.join(self.get_dir(), chall_name)
            logger.debug('Downloading ' +
                         self.get_remote_challenge_data_package() + ' to ' +
                         localfile)
            return ft.download_file(self.get_remote_challenge_data_package(),
                                    localfile)
        finally:
            try:
                ft.disconnect()
            except Exception:
                logger.exception('Caught exception trying to disconnect')

    def _runtask(self):
        """Performs the download and untar of challenge data package
        """
        chall_file = self._download_remote_challenge_data_package_with_retry()
        chall_name = self._untar_challenge_data_package(chall_file)
        self._move_challenge_data_package_into_task_dir(chall_name)
        summary = self._get_summary_of_docked_results()
        self.append_to_email_log(summary)

    def run(self):
        """Runs ExternalDataSubmission task

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(ExternalDataSubmissionTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return
        try:
            self._runtask()
        except Exception as e:
            logger.exception('Caught exception')
            self.set_error('Caught exception ' + str(e))

        # assess the result
        self.end()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_blastnfilter
----------------------------------

Tests for `blastnfilter` module.
"""

import unittest

from d3r import blastnfilter


class TestBlastnfilter(unittest.TestCase):

    def setUp(self):
        pass

    def test_parse_arguments(self):
        theargs = []
        try:
            blastnfilter._parse_arguments('hi', theargs)
            self.fail('expected exception')
        except:
            pass

        theargs = ['--outdir', 'myoutdir',
                   '--nonpolymertsv', 'mynonpolymertsv',
                   '--sequencetsv', 'mysequencetsv',
                   '--crystalpH', 'mycrystalph',
                   '--pdbblastdb', 'mypdbblastdb',
                   '--pdbdb', 'mypdbdb',
                   '--compinchi', 'mycompinchi']
        result = blastnfilter._parse_arguments('hi', theargs)
        self.assertEqual(result.out, 'myoutdir')
        self.assertEqual(result.non_polymer, 'mynonpolymertsv')
        self.assertEqual(result.loglevel, 'WARNING')
        self.assertEqual(result.polymer, 'mysequencetsv')
        self.assertEqual(result.ph, 'mycrystalph')
        self.assertEqual(result.blast_db, 'mypdbblastdb')
        self.assertEqual(result.pdb_path, 'mypdbdb')
        self.assertEqual(result.compinchi, 'mycompinchi')
        theargs.append('--log')
        theargs.append('DEBUG')
        result = blastnfilter._parse_arguments('hi', theargs)
        self.assertEqual(result.loglevel, 'DEBUG')

    def test_main_args_set_but_invalid(self):
        theargs = ['blastnfilter',
                   '--outdir', 'myoutdir',
                   '--nonpolymertsv', 'mynonpolymertsv',
                   '--sequencetsv', 'mysequencetsv',
                   '--crystalpH', 'mycrystalph',
                   '--pdbblastdb', 'mypdbblastdb',
                   '--pdbdb', 'mypdbdb',
                   '--compinchi', 'mycompinchi']
        try:
            blastnfilter.main(theargs)
            self.fail('expected exception')
        except:
            pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



__author__ = 'churas'

import unittest
import tempfile
import os.path

"""
test_chimeraprep
--------------------------------

Tests for `chimeraprep` module.
"""

import shutil
import os
from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.challengedata import ChallengeDataTask
from d3r.celpp.chimeraprep import ChimeraProteinLigPrepTask


class TestChimeraProteinLigPrepTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            # try with no dir
            self.assertEqual(task.get_uploadable_files(), [])

            # try with empty dir
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # try with final log
            final_log = os.path.join(task.get_dir(),
                                     ChimeraProteinLigPrepTask.FINAL_LOG)
            open(final_log, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            flist.index(final_log)

            # add error out files and try with second pbdid folder
            # with ligand.mae
            errfile = os.path.join(task.get_dir(),
                                   'chimera_proteinligprep.py.stderr')
            open(errfile, 'a').close()
            outfile = os.path.join(task.get_dir(),
                                   'chimera_proteinligprep.py.stdout')
            open(outfile, 'a').close()

            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(errfile)
            flist.index(outfile)
            flist.index(final_log)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # no blastnfilter task found so it cannot run
            params = D3RParameters()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'challengedata task has notfound status')

            # blastnfilter  running
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()
            open(os.path.join(chall.get_dir(), D3RTask.START_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'challengedata task has start status')

            # blastnfilter failed
            error_file = os.path.join(chall.get_dir(),
                                      D3RTask.ERROR_FILE)
            open(error_file, 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             'challengedata task has error status')

            # blastnfilter success
            os.remove(error_file)
            open(os.path.join(chall.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            self.assertEqual(task.can_run(), True)
            self.assertEqual(task.get_error(), None)

            # chimeraproteinligprep task exists already
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            task.create_dir()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(),
                             task.get_dir_name() +
                             ' already exists and status is unknown')

            # chimeraproteinligprep already complete
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            open(os.path.join(task.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEqual(task.can_run(), False)
            self.assertEqual(task.get_error(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            # return immediately cause can_run is false
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'challengedata task has notfound status')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_chimeraprep_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()
            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'chimeraprep not set')
            # test files get created
            self.assertEqual(os.path.isdir(task.get_dir()),
                             True)
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_pdbdb_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.chimeraprep = 'false'
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()
            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)
            task.run()
            self.assertEqual(task.get_error(),
                             'pdbdb not set')
            # test files get created
            self.assertEqual(os.path.isdir(task.get_dir()),
                             True)
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_chimeraprep_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.chimeraprep = 'false'
            params.pdbdb = '/foo'
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()
            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)

            task.run()
            self.assertEqual(task.get_error(),
                             'Non zero exit code: 1 received. Standard out: ' +
                             ' Standard error: ')
            # test file gets created
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)

            stderr = os.path.join(task.get_dir(),
                                  'false.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(task.get_dir(),
                                  'false.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_chimeraprep_is_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.chimeraprep = '/bin/doesnotexist'
            params.pdbdb = '/foo'
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()

            challdir = os.path.join(chall.get_dir(),
                                    chall.get_celpp_challenge_data_dir_name())

            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)

            task.run()
            self.assertEqual(task.get_error(),
                             'Caught Exception trying to run ' +
                             '/bin/doesnotexist --candidatedir ' +
                             challdir +
                             ' --rdkitpython \'\' ' +
                             '--pdbdb ' +
                             '/foo --outdir ' +
                             task.get_dir() +
                             ' : [Errno 2] No such file or directory')

            # test files get created
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_chimeraprep_is_not_found_rdkitpython_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.chimeraprep = '/bin/doesnotexist'
            params.pdbdb = '/foo'
            params.rdkitpython = '/data/miniconda2/bin'
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()

            challdir = os.path.join(chall.get_dir(),
                                    chall.get_celpp_challenge_data_dir_name())

            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)

            task.run()
            self.assertEqual(task.get_error(),
                             'Caught Exception trying to run ' +
                             '/bin/doesnotexist --candidatedir ' +
                             challdir +
                             ' --rdkitpython \'/data/miniconda2/bin\' ' +
                             '--pdbdb ' +
                             '/foo --outdir ' +
                             task.get_dir() +
                             ' : [Errno 2] No such file or directory')

            # test files get created
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.chimeraprep = 'true'
            params.pdbdb = '/foo'
            chall = ChallengeDataTask(temp_dir, params)
            chall.create_dir()
            open(os.path.join(chall.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            task = ChimeraProteinLigPrepTask(temp_dir, params)

            task.run()
            self.assertEqual(task.get_error(), None)
            # test files get created
            errfile = os.path.join(task.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(task.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(task.get_dir(),
                                  'true.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(task.get_dir(),
                                  'true.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = 'j5wagner'

import commands
import os
import glob
import logging
import time
import re 

from d3r.celppade.customeprep import Prep

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.DEBUG )


class OmegaPrep(Prep):

    # This prep script will be required to output files with the appropriate suffixes
    OUTPUT_PROTEIN_SUFFIX = '.pdb'
    OUTPUT_LIG_SUFFIX = '.sdf'



    def ligand_prepare(self, lig_smi_file, out_lig_file, info_dic={}):

        lig_prefix = os.path.basename(lig_smi_file).replace('.smi','')

        # Perform conformer generation using omega
        omega_stdout_file = lig_prefix + '_omega_confgen_stdout'
        omega_stderr_file = lig_prefix + '_omega_confgen_stderr'
        omega_cmd = 'omega2 -flipper true -in ' + lig_smi_file + ' -out ' + out_lig_file + ' 2> ' + omega_stderr_file + ' 1> ' + omega_stdout_file
        logging.info('Running omega command: ' + omega_cmd)
        commands.getoutput(omega_cmd)


        if not(os.path.isfile(out_lig_file)):
            logging.info('Prepared ligand file %s does not exist. Assuming that prep failed.' %(out_lig_file))
            return False
        else:
            return True




if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-p", "--pdbdb", metavar = "PATH", help = "PDB DATABANK which we will dock into")
    parser.add_argument("-c", "--challengedata", metavar="PATH", help = "PATH to the unpacked challenge data package")
    parser.add_argument("-o", "--prepdir", metavar = "PATH", help = "PATH to the output directory")
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level = logging.INFO )
    opt = parser.parse_args()
    pdb_location = opt.pdbdb
    challenge_data_path = opt.challengedata
    prep_result_path = opt.prepdir

    #running under this dir
    abs_running_dir = os.getcwd()
    log_file_path = os.path.join(abs_running_dir, 'final.log')
    log_file_dest = os.path.join(os.path.abspath(prep_result_path), 'final.log')

    OmegaPrep.proteinprep(challenge_data_path, pdb_location, prep_result_path)

    #move the final log file to the result dir
    commands.getoutput("mv %s %s"%(log_file_path, prep_result_path))




#!/usr/bin/env python
# -*- coding: utf-8 -*-


import unittest
import tempfile
import shutil
import os.path

from d3r.celpp.filetransfer import FileTransfer
from d3r.celpp.filetransfer import InvalidFtpConfigException
"""
test_uploader
----------------------------------

Tests for `uploader` module.
"""


class MockFtp(object):
    """Dummy ftp connection
    """


class TestFileTransfer(unittest.TestCase):

    def setUp(self):
        pass

    def test_filetransfer_parse_config(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # test _parse_config with None
            foo = FileTransfer(None)
            try:
                foo._parse_config(None)
                self.fail('Expected InvalidFtpConfigException')
            except InvalidFtpConfigException:
                pass

            # test passing invalid config file
            f = open(os.path.join(temp_dir, 'invalid'), 'a')
            f.write('hello\nhow\nare')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'invalid'))
            self.assertEqual(foo.get_host(), None)
            self.assertEqual(foo.get_password(), None)
            self.assertEqual(foo.get_user(), None)
            self.assertEqual(foo.get_remote_dir(), '')
            self.assertEqual(foo.get_remote_challenge_dir(), None)
            self.assertEqual(foo.get_remote_submission_dir(), None)

            # test passing partial config file
            f = open(os.path.join(temp_dir, 'partial'), 'a')
            f.write('#hello\nuser bob@bob.com\nare')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'partial'))
            self.assertEqual(foo.get_host(), None)
            self.assertEqual(foo.get_password(), None)
            self.assertEqual(foo.get_user(), 'bob@bob.com')
            self.assertEqual(foo.get_remote_dir(), '')
            self.assertEqual(foo.get_remote_challenge_dir(), None)
            self.assertEqual(foo.get_remote_submission_dir(), None)

            # test passing valid config file
            f = open(os.path.join(temp_dir, 'valid'), 'a')
            f.write('host ftp.box.com\nuser bob@bob.com\npass 222\npath /foo')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'valid'))
            self.assertEqual(foo.get_host(), 'ftp.box.com')
            self.assertEqual(foo.get_password(), '222')
            self.assertEqual(foo.get_user(), 'bob@bob.com')
            self.assertEqual(foo.get_remote_dir(), '/foo')
            self.assertEqual(foo.get_remote_challenge_dir(), None)
            self.assertEqual(foo.get_remote_submission_dir(), None)

            # test passing valid config file with challengepath
            f = open(os.path.join(temp_dir, 'valid'), 'a')
            f.write('host ftp.box.com\nuser bob@bob.com\npass 222\npath /foo\n'
                    'challengepath /chall\n')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'valid'))
            self.assertEqual(foo.get_host(), 'ftp.box.com')
            self.assertEqual(foo.get_password(), '222')
            self.assertEqual(foo.get_user(), 'bob@bob.com')
            self.assertEqual(foo.get_remote_dir(), '/foo')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
            self.assertEqual(foo.get_remote_submission_dir(), None)

            # test passing valid config file with challengepath and
            # submissionpath
            f = open(os.path.join(temp_dir, 'valid'), 'a')
            f.write('host ftp.box.com\nuser bob@bob.com\npass 222\npath /foo\n'
                    'challengepath /chall\nsubmissionpath /submit\n')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'valid'))
            self.assertEqual(foo.get_host(), 'ftp.box.com')
            self.assertEqual(foo.get_password(), '222')
            self.assertEqual(foo.get_user(), 'bob@bob.com')
            self.assertEqual(foo.get_remote_dir(), '/foo')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
            self.assertEqual(foo.get_remote_submission_dir(), '/submit')

            # test passing valid config file with challengepath,
            # submissionpath, and contestantid
            f = open(os.path.join(temp_dir, 'valid'), 'a')
            f.write('host ftp.box.com\nuser bob@bob.com\npass 222\npath /foo\n'
                    'challengepath /chall\nsubmissionpath /submit\n'
                    'contestantid 5678\n')
            f.flush()
            f.close()
            foo = FileTransfer(os.path.join(temp_dir, 'valid'))
            self.assertEqual(foo.get_host(), 'ftp.box.com')
            self.assertEqual(foo.get_password(), '222')
            self.assertEqual(foo.get_user(), 'bob@bob.com')
            self.assertEqual(foo.get_remote_dir(), '/foo')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
            self.assertEqual(foo.get_remote_submission_dir(), '/submit')
            self.assertEqual(foo.get_contestant_id(), '5678')

        finally:
            shutil.rmtree(temp_dir)

    def test_constructor(self):
        # test passing None which means we set everything manually
        foo = FileTransfer(None)

        self.assertEqual(foo.get_host(), None)
        self.assertEqual(foo.get_password(), None)
        self.assertEqual(foo.get_user(), None)
        self.assertEqual(foo.get_remote_dir(), '')
        self.assertEqual(foo.get_error_msg(), None)
        self.assertEqual(foo.get_remote_challenge_dir(), None)

        temp_dir = tempfile.mkdtemp()
        try:
            # test passing non existant config file
            try:
                foo = FileTransfer(os.path.join(temp_dir, 'doesnotexist'))
                self.fail('Expected IOError')
            except IOError:
                pass

            # test passing empty config file
            open(os.path.join(temp_dir, 'empty'), 'a').close()
            foo = FileTransfer(os.path.join(temp_dir, 'empty'))
            self.assertEqual(foo.get_host(), None)
            self.assertEqual(foo.get_password(), None)
            self.assertEqual(foo.get_user(), None)
            self.assertEqual(foo.get_remote_dir(), '')
            self.assertEqual(foo.get_remote_challenge_dir(), None)

            # test getters and setters
            foo.set_connect_timeout(10)
            foo.set_connection('hi')
            foo.set_host('host')
            foo.set_password('pass')
            foo.set_remote_dir('/remote')
            foo.set_user('user')
            foo.set_remote_challenge_dir('/chall')

            self.assertEqual(foo.get_connect_timeout(), 10)
            self.assertEqual(foo.get_host(), 'host')
            self.assertEqual(foo.get_password(), 'pass')
            self.assertEqual(foo.get_remote_dir(), '/remote')
            self.assertEqual(foo.get_user(), 'user')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
        finally:
            shutil.rmtree(temp_dir)

    def test_connect(self):
        foo = FileTransfer(None)
        self.assertTrue(foo.connect())

    def test_disconnect(self):
        foo = FileTransfer(None)
        foo.disconnect()

    def test_ftp_upload_file_direct(self):
        foo = FileTransfer(None)

        self.assertEqual(foo.upload_file_direct(None, '/remote_dir',
                                                'remote_filename'), False)

        self.assertEqual(foo.get_error_msg(),
                         'upload_file_direct not implemented')

    def test_upload_files(self):
        foo = FileTransfer(None)
        self.assertEqual(foo.upload_files(None), False)
        self.assertEqual(foo.get_error_msg(),
                         'upload_files not implemented')
        self.assertEqual(foo.get_upload_summary(),
                         'upload_files not implemented\n0 (0 bytes) files '
                         'uploaded in 0 seconds to host Unset:')

    def test_get_upload_summary(self):
        foo = FileTransfer(None)
        sum = foo.get_upload_summary()
        self.assertEqual(sum, '0 (0 bytes) files uploaded in 0 seconds to'
                              ' host Unset:')

    def test_delete_file_none_file(self):
        foo = FileTransfer(None)
        self.assertFalse(foo.delete_file(None))
        self.assertEqual(foo.get_error_msg(), 'delete_file not implemented')

    def test_download_file(self):
        foo = FileTransfer(None)
        self.assertFalse(foo.download_file(None, None))
        self.assertEqual(foo.get_error_msg(), 'download_file not implemented')

    def test_list_dirs(self):
        foo = FileTransfer(None)
        self.assertFalse(foo.list_dirs(None))
        self.assertEqual(foo.get_error_msg(), 'list_dirs not implemented')

    def test_list_files(self):
        foo = FileTransfer(None)
        self.assertFalse(foo.list_files(None))
        self.assertEqual(foo.get_error_msg(), 'list_files not implemented')

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'churas'


import unittest
import os
import tempfile
import shutil
from datetime import datetime
from dateutil.tz import tzutc
from dateutil.tz import tzlocal
from datetime import timedelta
from mock import Mock
from d3r.celpp.filetransfer import FtpFileTransfer


"""
test_dataimport
----------------------------------

Tests for `dataimport` module.
"""

from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.dataimport import ImportRetryCountExceededError
from d3r.celpp.makeblastdb import MakeBlastDBTask
from d3r.celpp import util

from tests.celpp import test_task


class TestDataImportTask(unittest.TestCase):
    def setUp(self):
        pass

    def get_total_seconds(self, td):
        return (td.microseconds +
                (td.seconds + td.days * 24 * 3600) * 10**6) / 10**6

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            self.assertEqual(task.get_uploadable_files(), [])

            task.create_dir()
            # test empty dir
            self.assertEqual(task.get_uploadable_files(), [])

            # test with only compinchi
            open(task.get_components_inchi_file(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 1)
            flist.index(task.get_components_inchi_file())

            # test with crystal file
            open(task.get_crystalph_tsv(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(task.get_components_inchi_file())
            flist.index(task.get_crystalph_tsv())

            # test with nonpolymer file
            open(task.get_nonpolymer_tsv(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 3)
            flist.index(task.get_components_inchi_file())
            flist.index(task.get_crystalph_tsv())
            flist.index(task.get_nonpolymer_tsv())

            # test with sequence file
            open(task.get_sequence_tsv(), 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 4)
            flist.index(task.get_components_inchi_file())
            flist.index(task.get_crystalph_tsv())
            flist.index(task.get_nonpolymer_tsv())
            flist.index(task.get_sequence_tsv())

        finally:
            shutil.rmtree(temp_dir)

    def test_update_status_from_filesystem(self):
        params = D3RParameters()
        task = DataImportTask(None, params)
        test_task.try_update_status_from_filesystem(self, task)

    def test_get_nonpolymer_tsv(self):
        params = D3RParameters()
        task = DataImportTask('/foo', params)
        self.assertEqual(task.get_nonpolymer_tsv(),
                         '/foo/' + task.get_dir_name() +
                         '/new_release_structure_nonpolymer.tsv')

    def test_get_sequence_tsv(self):
        params = D3RParameters()
        task = DataImportTask('/foo', params)
        self.assertEqual(task.get_sequence_tsv(),
                         '/foo/' + task.get_dir_name() +
                         '/new_release_structure_sequence_canonical.tsv')

    def test_get_crystalph_tsv(self):
        params = D3RParameters()
        task = DataImportTask('/foo', params)
        self.assertEqual(task.get_crystalph_tsv(),
                         '/foo/' + task.get_dir_name() +
                         '/new_release_crystallization_pH.tsv')

    def test_get_participant_list_csv(self):
        params = D3RParameters()
        task = DataImportTask('/foo', params)
        self.assertEqual(task.get_participant_list_csv(),
                         '/foo/' + task.get_dir_name() +
                         '/' + DataImportTask.PARTICIPANT_LIST_CSV)

    def test_append_standard_to_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            task.append_standard_to_files()
            self.assertTrue(os.path.isfile(task.get_nonpolymer_tsv()))
            self.assertTrue(os.path.isfile(task.get_sequence_tsv()))
            self.assertTrue(os.path.isfile(task.get_crystalph_tsv()))

            # now do it again, but this time make the append fail
            # cause the nonpolymer_tsv is a directory
            os.unlink(task.get_nonpolymer_tsv())
            os.makedirs(task.get_nonpolymer_tsv())
            task.append_standard_to_files()

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run_with_complete_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            open(os.path.join(task.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEquals(task.can_run(), False)
            self.assertEquals(task.get_error(), None)
            self.assertEquals(task._can_run, False)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run_does_not_exist_or_error(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)

            # no make blast db
            self.assertEquals(task.can_run(), False)
            self.assertEquals(task.get_error(),
                              'makeblastdb task has notfound status')
            self.assertEquals(task._can_run, False)

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()

            # make blast db failed
            err_file = os.path.join(make_blast.get_dir(),
                                    D3RTask.ERROR_FILE)
            open(err_file, 'a').close()
            self.assertEquals(task.can_run(), False)
            self.assertEquals(task.get_error(),
                              'makeblastdb task has error status')
            self.assertEquals(task._can_run, False)

            os.remove(err_file)

            # make blast db success
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            self.assertEquals(task.can_run(), True)
            self.assertEquals(task.get_error(), None)
            self.assertEquals(task._can_run, True)

            task.create_dir()
            open(os.path.join(task.get_dir(),
                              D3RTask.ERROR_FILE), 'a').close()
            self.assertEquals(task.can_run(), False)
            self.assertEquals(task._can_run, False)
            self.assertEquals(task.get_error(),
                              task.get_dir_name() + ' already exists and ' +
                              'status is ' + D3RTask.ERROR_STATUS)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_but_can_run_flag_is_false(self):
        params = D3RParameters()
        task = DataImportTask('/foo', params)
        task._can_run = False
        task.run()

    def test_run_pdbfileurl_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            task.run()
            self.assertEquals(task.get_error(), 'cannot download files' +
                              ' cause pdbfileurl not set')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_can_run_flag_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir
            task = DataImportTask(temp_dir, params)
            task._can_run = False
            task.run()
            self.assertEquals(task.get_error(), None)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_compinchi_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            task.run()
            self.assertEquals(task.get_error(), 'cannot download files' +
                              ' cause compinchi not set')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_nonpolymer_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            task.run()
            self.assertEquals(task.get_error(), 'Unable to download file ' +
                              'from ' + params.pdbfileurl + ' to ' +
                              task.get_nonpolymer_tsv())
        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_sequence_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            open(os.path.join(temp_dir,
                              task.NONPOLYMER_TSV), 'a').close()
            task.run()
            self.assertEquals(task.get_error(), 'Unable to download file ' +
                              'from ' + params.pdbfileurl + ' to ' +
                              task.get_sequence_tsv())

        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_crystalph_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            open(os.path.join(temp_dir,
                              task.NONPOLYMER_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.SEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.OLDSEQUENCE_TSV), 'a').close()
            task.run()
            self.assertEquals(task.get_error(), 'Unable to download file ' +
                              'from ' + params.pdbfileurl + ' to ' +
                              task.get_crystalph_tsv())

        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_compinchi_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            open(os.path.join(temp_dir,
                              task.NONPOLYMER_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.SEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.OLDSEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.CRYSTALPH_TSV), 'a').close()
            task.run()
            self.assertEquals(task.get_error(), 'Unable to download file ' +
                              'from ' + params.compinchi + ' to ' +
                              task.get_components_inchi_file())

        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_success_except_participant_download_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task._retrysleep = 0
            open(os.path.join(temp_dir,
                              task.NONPOLYMER_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.SEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.OLDSEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.CRYSTALPH_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.COMPINCHI_ICH), 'a').close()

            task.run()
            self.assertEquals(task.get_error(), None)

            # check line count is 1 now which indicates
            # standard was added
            self.assertEqual(util.get_file_line_count(
                task.get_nonpolymer_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_sequence_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_oldsequence_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_crystalph_tsv()), 1)

            self.assertTrue(task.get_email_log()
                            .startswith('\nWARNING: Unable to download'))
        finally:
            shutil.rmtree(temp_dir)

    def test_run_all_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            fakeftp = FtpFileTransfer(None)
            mftp = D3RParameters()

            fakeftp.set_connection(mftp)
            fakeftp.set_remote_dir('/foo2')
            mftp.get = Mock()

            params = D3RParameters()
            params.pdbfileurl = 'file://' + temp_dir
            params.compinchi = 'file://' + temp_dir

            make_blast = MakeBlastDBTask(temp_dir, params)
            make_blast.create_dir()
            open(os.path.join(make_blast.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            task = DataImportTask(temp_dir, params)
            task.set_file_transfer(fakeftp)
            task._retrysleep = 0
            open(os.path.join(temp_dir,
                              task.NONPOLYMER_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.SEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.OLDSEQUENCE_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.CRYSTALPH_TSV), 'a').close()
            open(os.path.join(temp_dir,
                              task.COMPINCHI_ICH), 'a').close()

            task.run()
            self.assertEquals(task.get_error(), None)

            # check line count is 1 now which indicates
            # standard was added
            self.assertEqual(util.get_file_line_count(
                task.get_nonpolymer_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_sequence_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_oldsequence_tsv()), 1)
            self.assertEqual(util.get_file_line_count(
                task.get_crystalph_tsv()), 1)

            mftp.get.assert_called_with('/foo2/' +
                                        DataImportTask.PARTICIPANT_LIST_CSV,
                                        local=task.get_participant_list_csv())
        finally:
            shutil.rmtree(temp_dir)

    def test_wait_for_url_to_be_updated(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # skipimportwait is None
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task._wait_for_url_to_be_updated('foo')

            # skipimportwait is True
            params = D3RParameters()
            params.skipimportwait = True
            task = DataImportTask(temp_dir, params)
            task._wait_for_url_to_be_updated('foo')

            params = D3RParameters()
            params.skipimportwait = False
            params.importretry = 2
            params.importsleep = 0
            task = DataImportTask(temp_dir, params)

            fakefile = os.path.join(temp_dir, 'foo')

            f = open(fakefile, 'w')
            f.write('hi\n')
            f.flush()
            f.close()

            prev_friday = util.get_previous_friday_from_date(
                datetime.now(tzlocal()))

            thurs = prev_friday - timedelta(days=1)
            dse = thurs - datetime(1970, 01, 01, tzinfo=tzutc())
            secs_since_epoch = self.get_total_seconds(dse)
            os.utime(fakefile, (secs_since_epoch, secs_since_epoch))
            # test retry exceeded
            try:
                task._wait_for_url_to_be_updated('file://' + fakefile)
                self.fail('Expected ImportRetryCountExceededError')
            except ImportRetryCountExceededError:
                pass

            # test importsleep not defined
            params = D3RParameters()
            params.skipimportwait = False
            params.importretry = 2
            task.set_args(params)
            try:
                task._wait_for_url_to_be_updated('file://' + fakefile)
                self.fail('Expected ImportRetryCountExceededError')
            except ImportRetryCountExceededError:
                pass

            sat = prev_friday + timedelta(days=1)
            dse = sat - datetime(1970, 01, 01, tzinfo=tzutc())
            secs_since_epoch = self.get_total_seconds(dse)
            os.utime(fakefile, (secs_since_epoch, secs_since_epoch))

            task._wait_for_url_to_be_updated('file://' + fakefile)

        finally:
            shutil.rmtree(temp_dir)

    def test_download_files_with_wait_all_successful(self):
        temp_dir = tempfile.mkdtemp()
        try:
            prev_friday = util.get_previous_friday_from_date(
                datetime.now(tzlocal()))
            sat = prev_friday + timedelta(days=1)
            dse = sat - datetime(1970, 01, 01, tzinfo=tzutc())
            secs_since_epoch = self.get_total_seconds(dse)

            nonpoly = os.path.join(temp_dir,
                                   DataImportTask.NONPOLYMER_TSV)
            f = open(nonpoly, 'w')
            f.write('nonpoly\n')
            f.flush()
            f.close()
            os.utime(nonpoly, (secs_since_epoch, secs_since_epoch))

            seq = os.path.join(temp_dir,
                               DataImportTask.SEQUENCE_TSV)
            f = open(seq, 'w')
            f.write('seq\n')
            f.flush()
            f.close()
            os.utime(seq, (secs_since_epoch, secs_since_epoch))

            oldseq = os.path.join(temp_dir,
                                  DataImportTask.OLDSEQUENCE_TSV)
            f = open(oldseq, 'w')
            f.write('oldseq\n')
            f.flush()
            f.close()
            os.utime(oldseq, (secs_since_epoch, secs_since_epoch))

            crystal = os.path.join(temp_dir,
                                   DataImportTask.CRYSTALPH_TSV)
            f = open(crystal, 'w')
            f.write('crystal\n')
            f.flush()
            f.close()
            os.utime(crystal, (secs_since_epoch, secs_since_epoch))

            comp = os.path.join(temp_dir, DataImportTask.COMPINCHI_ICH)

            f = open(comp, 'w')
            f.write('comp\n')
            f.flush()
            f.close()

            params = D3RParameters()
            params.skipimportwait = False
            params.importretry = 2
            params.importsleep = 0
            params.compinchi = 'file://' + temp_dir
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            val = task._download_files('file://' + temp_dir)
            self.assertEqual(task.get_error(), None)
            self.assertTrue(val)

        finally:
            shutil.rmtree(temp_dir)

    def test_download_files_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            nonpoly = os.path.join(temp_dir,
                                   DataImportTask.NONPOLYMER_TSV)
            f = open(nonpoly, 'w')
            f.write('nonpoly\n')
            f.flush()
            f.close()

            seq = os.path.join(temp_dir,
                               DataImportTask.SEQUENCE_TSV)
            f = open(seq, 'w')
            f.write('seq\n')
            f.flush()
            f.close()

            oldseq = os.path.join(temp_dir,
                                  DataImportTask.OLDSEQUENCE_TSV)
            f = open(oldseq, 'w')
            f.write('oldseq\n')
            f.flush()
            f.close()

            crystal = os.path.join(temp_dir,
                                   DataImportTask.CRYSTALPH_TSV)
            f = open(crystal, 'w')
            f.write('crystal\n')
            f.flush()
            f.close()

            params = D3RParameters()
            params.skipimportwait = True
            params.compinchi = 'file://' + temp_dir
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            val = task._download_files('file://' + temp_dir)
            self.assertEqual(task.get_error(), 'Unable to download file from '
                                               'file://' + temp_dir + ' to ' +
                             task.get_components_inchi_file())
            self.assertFalse(val)

        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_nonexistant_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            pdbid_set = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbid_set), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_empty_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            open(task.get_crystalph_tsv(), 'a').close()
            pdbid_set = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(os.path.isfile(task.get_crystalph_tsv()), True)
            self.assertEqual(len(pdbid_set), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_no_header_invalid_lines(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('hello\nx\n')
            f.flush()
            f.close()
            pdbid_set = task.get_set_of_pdbid_from_crystalph_tsv()
            self.assertEqual(os.path.isfile(task.get_crystalph_tsv()), True)
            self.assertEqual(len(pdbid_set), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_internal_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            open(task.get_crystalph_tsv(), 'a').close()
            os.chmod(task.get_crystalph_tsv(), 0o000)
            pdbid_set = task.get_set_of_pdbid_from_crystalph_tsv()
            self.assertEqual(len(pdbid_set), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_valid_entries(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('PDB_ID  _exptl_crystal_grow.pH\n')
            f.write('4rfr    7.5\n')
            f.write('4X09\t6.5\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()
            pdbid_set = task.get_set_of_pdbid_from_crystalph_tsv()
            self.assertEqual(os.path.isfile(task.get_crystalph_tsv()), True)
            self.assertEqual(len(pdbid_set), 5)
            self.assertEqual('4RFR' in pdbid_set, True)
            self.assertEqual('4X09' in pdbid_set, True)
            self.assertEqual('4XET' in pdbid_set, True)
            self.assertEqual('4XF1' in pdbid_set, True)
            self.assertEqual('4XF3' in pdbid_set, True)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_from_crystalph_tsv_invalid_entries(self):
        """Header missing and 4rfr has 3 columns
        """
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('4X09\t6.5\n')
            f.write('4rfr 7.5 8\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()
            pdbid_set = task.get_set_of_pdbid_from_crystalph_tsv()
            self.assertEqual(os.path.isfile(task.get_crystalph_tsv()), True)
            self.assertEqual(len(pdbid_set), 3)
            self.assertEqual('4RFR' in pdbid_set, False)
            self.assertEqual('4X09' in pdbid_set, False)
            self.assertEqual('4XET' in pdbid_set, True)
            self.assertEqual('4XF1' in pdbid_set, True)
            self.assertEqual('4XF3' in pdbid_set, True)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres_no_seq(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('PDB_ID  _exptl_crystal_grow.pH\n')
            f.write('4X09\t6.5\n')
            f.write('4rfr\t8\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()
            pdbset = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres_no_tsv(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            makeblast = MakeBlastDBTask(temp_dir, params)
            makeblast.create_dir()
            f = open(makeblast.get_pdb_seqres_txt(), 'w')
            f.write('>101m_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRVK'
                    'HLKTEAEMKASEDLKKHG\n')
            f.write('>102l_A mol:protein length:165  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAAKSELDKA'
                    'IGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRAAL'
                    'INMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRV'
                    'ITTFRTGTWDAYKNL\n')
            f.flush()
            f.close()

            pdbset = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres_empty_seq(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('PDB_ID  _exptl_crystal_grow.pH\n')
            f.write('4X09\t6.5\n')
            f.write('4rfr\t8\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()

            makeblast = MakeBlastDBTask(temp_dir, params)
            makeblast.create_dir()
            open(makeblast.get_pdb_seqres_txt(), 'a').close()

            pdbset = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres_no_hits(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('PDB_ID  _exptl_crystal_grow.pH\n')
            f.write('4X09\t6.5\n')
            f.write('4rfr\t8\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()

            makeblast = MakeBlastDBTask(temp_dir, params)
            makeblast.create_dir()
            f = open(makeblast.get_pdb_seqres_txt(), 'w')
            f.write('>101m_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRVK'
                    'HLKTEAEMKASEDLKKHG\n')
            f.write('>102l_A mol:protein length:165  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAAKSELDKA'
                    'IGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRAAL'
                    'INMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRV'
                    'ITTFRTGTWDAYKNL\n')
            f.flush()
            f.close()

            pdbset = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbset), 0)
        finally:
            shutil.rmtree(temp_dir)

    def test_get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres_w_hits(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            f = open(task.get_crystalph_tsv(), 'w')
            f.write('PDB_ID  _exptl_crystal_grow.pH\n')
            f.write('4X09\t6.5\n')
            f.write('4rfr\t8\n')
            f.write('4XET\t6.2\n')
            f.write('4XF1\t6.2\n')
            f.write('4XF3\t6.2\n')
            f.flush()
            f.close()

            makeblast = MakeBlastDBTask(temp_dir, params)
            makeblast.create_dir()
            f = open(makeblast.get_pdb_seqres_txt(), 'w')
            f.write('>4rfr_A mol:protein length:154  MYOGLOBIN\n')
            f.write('MVLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRVK'
                    'HLKTEAEMKASEDLKKHG\n')
            f.write('>102l_A mol:protein length:165  T4 LYSOZYME\n')
            f.write('MNIFEMLRIDEGLRLKIYKDTEGYYTIGIGHLLTKSPSLNAAAKSELDKA'
                    'IGRNTNGVITKDEAEKLFNQDVDAAVRGILRNAKLKPVYDSLDAVRRAAL'
                    'INMVFQMGETGVAGFTNSLRMLQQKRWDEAAVNLAKSRWYNQTPNRAKRV'
                    'ITTFRTGTWDAYKNL\n')
            f.flush()
            f.close()

            pdbset = task.get_set_of_pdbid_in_crystalph_tsv_and_pdb_seqres()
            self.assertEqual(len(pdbset), 1)
            self.assertEqual('4RFR' in pdbset, True)
        finally:
            shutil.rmtree(temp_dir)

    def test_download_participant_list_no_filetransfer(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.create_dir()
            task._download_participant_list_csv()
            self.assertEqual(task.get_email_log(),
                             '\nWARNING: Unable to download '
                             'participant_list.csv which means '
                             'external users will NOT get evaluation '
                             'email : \'NoneType\' object has no attribute '
                             '\'connect\'\n')
        finally:
            shutil.rmtree(temp_dir)

    def test_download_participant_list_file_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = FtpFileTransfer(None)
            mockftp = D3RParameters()
            mockftp.get = Mock()
            foo.set_remote_dir('/foo')
            foo.set_connection(mockftp)

            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.set_file_transfer(foo)
            task.create_dir()
            task._download_participant_list_csv()
            self.assertEqual(task.get_email_log(),
                             '\nWARNING: participant_list.csv not downloaded '
                             'which means external users will NOT get '
                             'evaluation email\n')
            mockftp.get\
                .assert_called_with('/foo/' +
                                    DataImportTask.PARTICIPANT_LIST_CSV,
                                    local=task.get_participant_list_csv())
        finally:
            shutil.rmtree(temp_dir)

    def test_download_participant_list_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = FtpFileTransfer(None)
            mockftp = D3RParameters()
            mockftp.get = Mock()
            foo.set_remote_dir('/foo')
            foo.set_connection(mockftp)

            params = D3RParameters()
            task = DataImportTask(temp_dir, params)
            task.set_file_transfer(foo)
            task.create_dir()
            open(task.get_participant_list_csv(), 'a').close()
            task._download_participant_list_csv()
            self.assertEqual(task.get_email_log(), None)
            mockftp.get\
                .assert_called_with('/foo/' +
                                    DataImportTask.PARTICIPANT_LIST_CSV,
                                    local=task.get_participant_list_csv())
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()
